from django.core.exceptions import ValidationError
from django.db import models

from ISOF.model.city import City
from ISOF.model.simple_model_query_set import SimpleModelQuerySet
from ISOF.model.web_service.smp_web_service import SmpWebService


class BlockQuerySet(SimpleModelQuerySet):
    #fetch all
    def _fetch_all(self):
        if (not hasattr(self, '_result_cache')) or (self._result_cache is None):
            self._result_cache = Block.get_all_blocks()
            self.create_dictionary(self._result_cache)

    def filter(self, *args, **kwargs):
        if (len(args) > 0):
            blocks = []
            args_part = args[0].children[0]
            if (isinstance(args_part, tuple) and isinstance(args_part[0], str) and (args_part[0] == 'city__in')):
                # TODO: Handle the entire list of cities.
                city = args_part[1][0]
                self._fetch_all()
                if (self._result_cache is not None):
                    for block in self._result_cache:
                        if (city.id == block.city_id):
                            blocks.append(block)
                queryset = BlockQuerySet(Block)
                queryset._result_cache = blocks
                queryset.create_dictionary(queryset._result_cache)
                return queryset
            else:
                search_string = args[0].children[0].children[0][1]
                if (search_string):
                    self._fetch_all()
                    if (self._result_cache is not None):
                        for block in self._result_cache:
                            if (search_string in block.name):
                                blocks.append(block)

                    queryset = BlockQuerySet(Block)
                    queryset._result_cache = blocks
                    queryset.create_dictionary(queryset._result_cache)
                    return queryset
        return self


class BlockManager(models.Manager):
    def get_queryset(self):
        queryset = BlockQuerySet(Block)
        return queryset

class Block(models.Model):
    all_blocks = None
    name_of_block = models.CharField(max_length=512, null=False, blank=False)
    city = models.ForeignKey(City, on_delete=models.CASCADE, null=False, blank=False, verbose_name="Stad")
    objects = BlockManager()

    class Meta:
        base_manager_name = 'objects'
        managed = True
        verbose_name = "kvarter"
        verbose_name_plural = "kvarter"

    def __str__(self):
        return self.name_of_block

    #get all blocks class method
    @classmethod
    def get_all_blocks(cls):
        #get blocks from webservice
        if Block.all_blocks is None:
            web_service = SmpWebService()
            blocks_json = web_service.get_blocks()
            blocks = Block.get_blocks_from_json(blocks_json)
            Block.all_blocks = blocks
        return Block.all_blocks

    #get blocks from json
    @classmethod
    def get_blocks_from_json(cls, blocks_json):
        blocks = []
        cities = City.objects.all()
        for block_json in blocks_json:
            block = Block()
            block.id = block_json['id']
            block.pk = block_json['id']
            block.name_of_block = block_json['name_of_block']
            block.city = cities.get(id=block_json['city'])
            blocks.append(block)
        return blocks

    #get block from json
    @classmethod
    def get_block_from_json(cls, block_json):
        block = Block()
        cities = City.objects.all()
        block.id = block_json['id']
        block.name_of_block = block_json['name_of_block']
        block.city = cities.get(id=block_json['city'])
        block.pk = block_json['id']
        return block

    #create block
    @classmethod
    def create(cls, block):
        web_service = SmpWebService()
        block_json = web_service.create_block(block, block.author.token, block.csrftoken)
        block = Block.get_block_from_json(block_json)
        Block.all_blocks = None
        return block

    #save
    def save(self, commit=True):
        web_service = SmpWebService()
        block_json = web_service.update_block(self, self.author.token, self.csrftoken)
        block = Block.get_block_from_json(block_json)
        Block.all_blocks = None
        return block

    def delete(self):
        web_service = SmpWebService()
        ok = web_service.delete_block(self, self.author.token, self.csrftoken)
        Block.all_blocks = None
        return ok

    #validate unique
    def validate_unique(self, exclude=None):
        blocks = Block.objects.all()
        for block in blocks:
            if(self.name_of_block == block.name_of_block and self.city == block.city):
                if (hasattr(self, 'id')) or (self.id is not None):
                    if(self.id != block.id):
                        raise ValidationError('Kvarteret finns redan i staden')
                else:
                    raise ValidationError('Kunde inte spara kvarter')

