from django.db import models

from ISOF.model.lexical_name import LexicalName
from ISOF.model.simple_model_query_set import SimpleModelQuerySet
from ISOF.model.web_service.smp_web_service import SmpWebService


class FormCatalogQuerySet(SimpleModelQuerySet):
    def _fetch_all(self):
        if (not hasattr(self, '_result_cache')) or (self._result_cache is None):
            self._result_cache = FormCatalog.get_all_form_catalogs()
            self.create_dictionary(self._result_cache)

    def filter(self, *args, **kwargs):
        if len(args) > 0:
            search_string = args[0].children[0].children[0][1]
            if search_string:
                form_catalogs = []
                self._fetch_all()
                if self._result_cache is not None:
                    for form_catalog in self._result_cache:
                        if search_string.casefold() in form_catalog.lexical_name.name.casefold():
                            form_catalogs.append(form_catalog)
                queryset = FormCatalogQuerySet(FormCatalog)
                queryset._result_cache = form_catalogs
                queryset.create_dictionary(queryset._result_cache)
                return queryset
        return self

    def get_form_catalogs_by_lexical_name(self, lexical_name):
        form_catalogs = []
        self._fetch_all()
        if self._result_cache is not None:
            for form_catalog in self._result_cache:
                if form_catalog.lexical_name == lexical_name:
                    form_catalogs.append(form_catalog)
        queryset = FormCatalogQuerySet(FormCatalog)
        queryset._result_cache = form_catalogs
        return queryset

    def get_form_catalogs_by_lexical_name_id(self, lexical_name_id):
        form_catalogs = []
        self._fetch_all()
        if (self._result_cache is not None):
            for form_catalog in self._result_cache:
                if (form_catalog.lexical_name.id == lexical_name_id):
                    form_catalogs.append(form_catalog)
        queryset = FormCatalogQuerySet(FormCatalog)
        queryset._result_cache = form_catalogs
        return queryset


# form catalog manager
class FormCatalogManager(models.Manager):
    def get_queryset(self):
        queryset = FormCatalogQuerySet(FormCatalog)
        return queryset


class FormCatalog(models.Model):
    all_form_catalogs = None
    external_note = models.TextField(max_length=4000, null=True, blank=True, verbose_name="Extern kommentar")
    internal_note = models.TextField(max_length=4000, null=True, blank=True, verbose_name="Intern kommentar")
    lexical_name = models.ForeignKey(LexicalName, null=False, blank=False, editable=True,
                                     on_delete=models.DO_NOTHING, verbose_name="Lexikalt namn")
    objects = FormCatalogManager()

    class Meta:
        base_manager_name = 'objects'
        managed = True
        verbose_name = "formkatalog"
        verbose_name_plural = "formkatalog"
        ordering = ['lexical_name__name']

    def __str__(self):
        return self.lexical_name.name

    @classmethod
    def get_all_form_catalogs(cls):
        if FormCatalog.all_form_catalogs is None:
            smp_service = SmpWebService()
            form_catalogs_json = smp_service.get_form_catalogs()
            form_catalogs = FormCatalog.get_form_catalogs_from_json(form_catalogs_json)
            FormCatalog.all_form_catalogs = form_catalogs
        return FormCatalog.all_form_catalogs

    @classmethod
    def get_form_catalogs_from_json(cls, form_catalogs_json):
        form_catalogs = []
        lexical_names = LexicalName.objects.all()
        for form_catalog_json in form_catalogs_json:
            form_catalog = FormCatalog()
            form_catalog.external_note = form_catalog_json['external_note']
            form_catalog.internal_note = form_catalog_json['internal_note']
            form_catalog.lexical_name = lexical_names.get(id=form_catalog_json['lexical_name'])
            form_catalog.id = form_catalog_json['id']
            form_catalog.pk = form_catalog_json['id']
            form_catalogs.append(form_catalog)
        return form_catalogs

    @classmethod
    def get_form_catalog_from_json(cls, form_catalog_json):
        form_catalog = FormCatalog()
        if form_catalog_json['external_note'] is not None:
            form_catalog.external_note = form_catalog_json['external_note']
        if form_catalog_json['internal_note'] is not None:
            form_catalog.internal_note = form_catalog_json['internal_note']
        form_catalog.lexical_name = LexicalName.objects.get(id=form_catalog_json['lexical_name'])
        form_catalog.id = form_catalog_json['id']
        form_catalog.pk = form_catalog_json['id']
        return form_catalog

    @classmethod
    def get_form_catalog_by_id(cls, form_catalog_id):
        form_catalogs = FormCatalog.get_all_form_catalogs()
        for form_catalog in form_catalogs:
            if form_catalog.id == form_catalog_id:
                return form_catalog
        return None

    @classmethod
    def get_form_catalogs_by_lexical_name(cls, lexical_name):
        form_catalogs = []
        form_catalogs_all = FormCatalog.get_all_form_catalogs()
        for form_catalog in form_catalogs_all:
            if form_catalog.lexical_name == lexical_name:
                form_catalogs.append(form_catalog)
        return form_catalogs

    @classmethod
    def get_form_catalogs_by_lexical_name_id(cls, lexical_name_id):
        form_catalogs = []
        form_catalogs_all = FormCatalog.get_all_form_catalogs()
        for form_catalog in form_catalogs_all:
            if form_catalog.lexical_name.id == lexical_name_id:
                form_catalogs.append(form_catalog)
        return form_catalogs

    #save
    def save(self, commit=True):
        smp_service = SmpWebService()
        form_catalog_json = smp_service.update_form_catalog(self, self.author.token, self.csrftoken)
        FormCatalog.all_form_catalogs = None
        return form_catalog_json

    #delete
    def delete(self):
        smp_service = SmpWebService()
        ok = smp_service.delete_form_catalog(self, self.author.token, self.csrftoken)
        FormCatalog.all_form_catalogs = None
        return ok

    #create
    @classmethod
    def create(cls, form_catalog):
        smp_service = SmpWebService()
        form_catalog_json = smp_service.create_form_catalog(form_catalog, form_catalog.author.token, form_catalog.csrftoken)
        new_form_catalog = FormCatalog.get_form_catalog_from_json(form_catalog_json)
        FormCatalog.all_form_catalogs = None
        return new_form_catalog

