from django.db import models

from ISOF.model.case import Case
from ISOF.model.form_catalog import FormCatalog
from ISOF.model.name_last_element import NameLastElement
from ISOF.model.simple_model_query_set import SimpleModelQuerySet
from ISOF.model.web_service.smp_web_service import SmpWebService


class FormCatalogNameQuerySet(SimpleModelQuerySet):
    def _fetch_all(self):
        if (not hasattr(self, '_result_cache')) or (self._result_cache is None):
            self._result_cache = FormCatalogName.get_all_form_catalog_names()
            self.create_dictionary(self._result_cache)

    def filter(self, *args, **kwargs):
        if len(args) > 0:
            search_string = args[0].children[0].children[0][1]
            if search_string:
                form_catalog_names = []
                self._fetch_all()
                if self._result_cache is not None:
                    for form_catalog_name in self._result_cache:
                        if search_string.casefold() in form_catalog_name.name.casefold():
                            form_catalog_names.append(form_catalog_name)
                queryset = FormCatalogNameQuerySet(FormCatalogName)
                queryset._result_cache = form_catalog_names
                queryset.create_dictionary(queryset._result_cache)
                return queryset
        return self

    def get_form_catalog_names_by_form_catalog(self, form_catalog):
        form_catalog_names = []
        self._fetch_all()
        if self._result_cache is not None:
            for form_catalog_name in self._result_cache:
                if form_catalog_name.form_catalog == form_catalog:
                    form_catalog_names.append(form_catalog_name)
        queryset = FormCatalogNameQuerySet(FormCatalogName)
        queryset._result_cache = form_catalog_names
        return queryset

    def get_form_catalog_names_by_form_catalog_id(self, form_catalog_id):
        form_catalog_names = []
        self._fetch_all()
        if self._result_cache is not None:
            for form_catalog_name in self._result_cache:
                if form_catalog_name.form_catalog.id == form_catalog_id:
                    form_catalog_names.append(form_catalog_name)
        queryset = FormCatalogNameQuerySet(FormCatalogName)
        queryset._result_cache = form_catalog_names
        return queryset


class FormCatalogNameManager(models.Manager):
    def get_queryset(self):
        queryset = FormCatalogNameQuerySet(FormCatalogName)
        return queryset


class FormCatalogName(models.Model):
    all_form_catalog_names = None
    case = models.ForeignKey(Case, null=False, blank=False,
                             on_delete=models.DO_NOTHING, verbose_name="Kasus")

    form_catalog = models.ForeignKey(FormCatalog, null=False, blank=False,
                                     on_delete=models.DO_NOTHING, verbose_name="Formkatalog")
    last_element = models.ForeignKey(NameLastElement, null=True, blank=True,
                                     on_delete=models.DO_NOTHING, verbose_name="Efterled")
    name = models.CharField(max_length=256, null=False, blank=False, verbose_name="Namn")
    start_year = models.CharField(max_length=128, null=False, blank=False, verbose_name="Startår")
    end_year = models.CharField(max_length=128, null=True, blank=True, verbose_name="Slutår")
    objects = FormCatalogNameManager()

    class Meta:
        verbose_name = "Formkatalognamn"
        verbose_name_plural = "Formkatalognamn"
        ordering = ['name']
        base_manager_name = 'objects'
        managed = True

    def __str__(self):
        return self.name

    @classmethod
    def get_all_form_catalog_names(cls):
        if FormCatalogName.all_form_catalog_names is None:
            smp_service = SmpWebService()
            form_catalog_names_json = smp_service.get_form_catalog_names()
            form_catalog_names = FormCatalogName.get_form_catalog_names_from_json(form_catalog_names_json)
            FormCatalogName.all_form_catalog_names = form_catalog_names
        return FormCatalogName.all_form_catalog_names

    @classmethod
    def get_form_catalog_names_from_json(cls, form_catalog_names_json):
        form_catalog_names = []
        form_catalogs = FormCatalog.objects.all()
        name_last_elements = NameLastElement.objects.all()
        cases = Case.objects.all()
        for form_catalog_name_json in form_catalog_names_json:
            form_catalog_name = FormCatalogName()
            form_catalog_name.case = cases.get(id=form_catalog_name_json['case'])
            if form_catalog_name_json['end_year'] is not None:
                form_catalog_name.end_year = form_catalog_name_json['end_year']
            form_catalog_name.form_catalog = form_catalogs.get(id=form_catalog_name_json['form_catalog'])
            form_catalog_name.id = form_catalog_name_json['id']
            form_catalog_name.pk = form_catalog_name_json['id']
            if form_catalog_name_json['last_element'] is not None:
                form_catalog_name.last_element = name_last_elements.get(id=form_catalog_name_json['last_element'])
            form_catalog_name.name = form_catalog_name_json['name']
            form_catalog_name.start_year = form_catalog_name_json['start_year']
            form_catalog_names.append(form_catalog_name)
        return form_catalog_names

    @classmethod
    def get_form_catalog_name_from_json(cls, form_catalog_name_json):
        form_catalog_name = FormCatalogName()
        cases = Case.objects.all()
        form_catalogs = FormCatalog.objects.all()
        last_elements = NameLastElement.objects.all()
        form_catalog_name.case = cases.get(id=form_catalog_name_json['case'])
        form_catalog_name.end_year = form_catalog_name_json['end_year']
        form_catalog_name.form_catalog = form_catalogs.get(id=form_catalog_name_json['form_catalog'])
        form_catalog_name.id = form_catalog_name_json['id']
        form_catalog_name.pk = form_catalog_name_json['id']
        if form_catalog_name_json['last_element'] is not None:
            form_catalog_name.last_element = last_elements.get(id=form_catalog_name_json['last_element'])
        form_catalog_name.name = form_catalog_name_json['name']
        form_catalog_name.start_year = form_catalog_name_json['start_year']
        return form_catalog_name

    @classmethod
    def get_form_catalog_name_by_id(cls, form_catalog_name_id):
        form_catalog_names = FormCatalogName.get_all_form_catalog_names()
        for form_catalog_name in form_catalog_names:
            if form_catalog_name.id == form_catalog_name_id:
                return form_catalog_name
        return None

    @classmethod
    def get_form_catalog_names_by_case(cls, case):
        form_catalog_names = []
        form_catalog_names_all = FormCatalogName.get_all_form_catalog_names()
        for form_catalog_name in form_catalog_names_all:
            if form_catalog_name.case == case:
                form_catalog_names.append(form_catalog_name)
        return form_catalog_names

    #create
    @classmethod
    def create(cls, form_catalog_name):
        smp_service = SmpWebService()
        form_catalog_name_json = smp_service.create_form_catalog_name(form_catalog_name, form_catalog_name.author.token,
                                                                      form_catalog_name.csrftoken)
        FormCatalogName.all_form_catalog_names = None
        return FormCatalogName.get_form_catalog_name_from_json(form_catalog_name_json)

    def save(self, commit=True):
        smp_service = SmpWebService()
        form_catalog_name_json = smp_service.update_form_catalog_name(self, self.author.token, self.csrftoken)
        FormCatalogName.all_form_catalog_names = None
        return form_catalog_name_json

    #delete
    def delete(self):
        smp_service = SmpWebService()
        ok = smp_service.delete_form_catalog_name(self, self.author.token, self.csrftoken)
        FormCatalogName.all_form_catalog_names = None
        return ok


