from ISOF.model.place_name_search_criteria import PlaceNameSearchCriteria
from ISOF.model.web_service.place_name_web_service import PlaceNameWebService
import threading

class GeoReferenceManager():
    counties = None
    districts = None
    lock = threading.Lock()
    parishes = None
    provinces = None

    def __init__(self):
        self.data_source = PlaceNameWebService()

    def get_counties(self):
        self.__load_data()
        return GeoReferenceManager.counties

    def get_county(self, county_id):
        for county in self.get_counties():
            if (county_id == county.id):
                return county

    def get_district(self, district_id):
        for district in self.get_districts():
            if (district_id == district.id):
                return district

    def get_districts(self):
        self.__load_data()
        return GeoReferenceManager.districts

    def get_parish(self, parish_id):
        for parish in self.get_parishes():
            if (parish_id == parish.id):
                return parish

    def get_parishes(self):
        self.__load_data()
        return GeoReferenceManager.parishes

    def get_place_name_search_criteria_counties(self, place_name_search_criteria):
        counties = ''

        if (hasattr(place_name_search_criteria, 'county_id')):
            if (isinstance(place_name_search_criteria.county_id, int)):
                county = self.get_county(place_name_search_criteria.county_id)
                if (counties != ''):
                    counties = counties + ', '
                counties = counties + county.name

            if (isinstance(place_name_search_criteria.county_id, list)):
                for county_id in place_name_search_criteria.county_id:
                    county = self.get_county(county_id)
                    if (counties != ''):
                        counties = counties + ', '
                    counties = counties + county.name

        return counties

    def get_place_name_search_criteria_districts(self, place_name_search_criteria):
        districts = ''

        if (hasattr(place_name_search_criteria, 'district_id')):
            if (isinstance(place_name_search_criteria.district_id, int)):
                district = self.get_district(place_name_search_criteria.district_id)
                if (districts != ''):
                    districts = districts + ', '
                districts = districts + district.name

            if (isinstance(place_name_search_criteria.district_id, list)):
                for district_id in place_name_search_criteria.district_id:
                    district = self.get_district(district_id)
                    if (districts != ''):
                        districts = districts + ', '
                    districts = districts + district.name

        return districts

    def get_place_name_search_criteria_parishes(self, place_name_search_criteria):
        parishes = ''

        if (hasattr(place_name_search_criteria, 'parish_id')):
            if (isinstance(place_name_search_criteria.parish_id, int)):
                parish = self.get_parish(place_name_search_criteria.parish_id)
                if (parishes != ''):
                    parishes = parishes + ', '
                parishes = parishes + parish.name

            if (isinstance(place_name_search_criteria.parish_id, list)):
                for parish_id in place_name_search_criteria.parish_id:
                    parish = self.get_parish(parish_id)
                    if (parishes != ''):
                        parishes = parishes + ', '
                    parishes = parishes + parish.name

        return parishes

    def get_place_name_search_criteria_provinces(self, place_name_search_criteria):
        provinces = ''

        if (hasattr(place_name_search_criteria, 'province_id')):
            if (isinstance(place_name_search_criteria.province_id, int)):
                province = self.get_province(place_name_search_criteria.province_id)
                if (provinces != ''):
                    provinces = provinces + ', '
                provinces = provinces + province.name

            if (isinstance(place_name_search_criteria.province_id, list)):
                for province_id in place_name_search_criteria.province_id:
                    province = self.get_province(province_id)
                    if (provinces != ''):
                        provinces = provinces + ', '
                    provinces = provinces + province.name

        return provinces

    def get_province(self, province_id):
        for province in self.get_provinces():
            if (province_id == province.id):
                return province

    def get_provinces(self):
        self.__load_data()
        return GeoReferenceManager.provinces

    def __load_data(self):
        GeoReferenceManager.lock.acquire()
        try:
            is_init = False
            if (GeoReferenceManager.provinces is None):
                is_init = True
                GeoReferenceManager.provinces = self.data_source.get_provinces()
            if (GeoReferenceManager.counties is None):
                is_init = True
                GeoReferenceManager.counties = self.data_source.get_counties()
            if (GeoReferenceManager.districts is None):
                GeoReferenceManager.districts = self.data_source.get_districts()
            if (GeoReferenceManager.parishes is None):
                GeoReferenceManager.parishes = self.data_source.get_parishes()

            if (is_init):
                for county in GeoReferenceManager.counties:
                    county.provinces = {}
                    for province_id in county.province_ids:
                        for province in GeoReferenceManager.provinces:
                            if (province_id == province.id):
                                county.provinces[province_id] = province

                for province in GeoReferenceManager.provinces:
                    province.counties = {}
                    for county_id in province.county_ids:
                        for county in GeoReferenceManager.counties:
                            if (county_id == county.id):
                                province.counties[county_id] = county

        finally:
            GeoReferenceManager.lock.release()
