from django.core.exceptions import ValidationError
from django.db import models

from ISOF.model.lexical_name import LexicalName
from ISOF.model.simple_model_query_set import SimpleModelQuerySet
from ISOF.model.web_service.smp_web_service import SmpWebService

from ISOF.model.language_smp import Language


class LexicalFormQuerySet(SimpleModelQuerySet):
    def remove(self, *objs, bulk=True):
        pass

    def delete(self):
        pass

    def _fetch_all(self):
        if (not hasattr(self, '_result_cache')) or (self._result_cache is None):
            self._result_cache = LexicalForm.get_all_lexical_forms()
            self.create_dictionary(self._result_cache)

    def filter(self, *args, **kwargs):
        lexical_name = None
        if (len(args) > 0):
            filter_tuple = args[0].children[0]
            if hasattr(filter_tuple, 'children'):
                search_string = filter_tuple.children[0][1]
                if (search_string):
                    lexical_forms = []
                    self._fetch_all()
                    if (self._result_cache is not None):
                        for lexical_form in self._result_cache:
                            if (search_string in lexical_form.form_type
                                    or search_string in lexical_form.name
                                    or search_string in lexical_form.language.language
                                    or search_string in lexical_form.lexical_name):
                                lexical_forms.append(lexical_form)
                    queryset = LexicalFormQuerySet(LexicalForm)
                    queryset._result_cache = lexical_forms
                    queryset.create_dictionary(queryset._result_cache)
                    return queryset

            else:
                lexical_name = filter_tuple[1][0]


        if (kwargs and 'lexical_name' in kwargs):
            lexical_name = kwargs['lexical_name']

        if (lexical_name):
            lexical_forms = []
            self._fetch_all()
            if (self._result_cache is not None):
                for lexical_form in self._result_cache:
                    if (lexical_form.lexical_name_id == lexical_name.id):
                        lexical_forms.append(lexical_form)
            queryset = LexicalFormQuerySet(LexicalForm)
            queryset._result_cache = lexical_forms
            return queryset

        return self


class LexicalFormManager(models.Manager):
    def get_queryset(self):
        queryset = LexicalFormQuerySet(LexicalForm)
        return queryset


class LexicalForm(models.Model):
    all_lexical_forms = None
    # add all fields of LexicalForm
    FORM_TYPE = (
        ('MainForm', 'Huvudform'),
        ('SecondaryForm', 'Sidoform'),
        ('DoubtfulForm', 'Osäker form'),
        ('ReconstructedForm', 'Rekonstruerad form')
    )
    form_type = models.CharField(max_length=20, null=False, blank=False, choices=FORM_TYPE, verbose_name="Formtyp")
    language = models.ForeignKey(Language, null=False, blank=False, editable=True,
                                 on_delete=models.DO_NOTHING, limit_choices_to={'is_in_lexical_name': True}, verbose_name="Språk", default=96)
    lexical_name = models.ForeignKey(LexicalName, null=True, blank=True, editable=True, on_delete=models.CASCADE, verbose_name="Lexikalt namn")
    is_in_book = models.BooleanField(default=True, null=False, verbose_name="Finns i bok")
    change_date = models.DateTimeField(auto_now=True, null=False)
    name = models.CharField(max_length=100, null=False, blank=False, verbose_name="Namn")
    objects = LexicalFormManager()

    # meta class
    class Meta:
        base_manager_name = 'objects'
        managed = True
        verbose_name = "lexikal form"
        verbose_name_plural = "lexikala former"

    def __str__(self):
        return "Lexikal form: " + self.name

    @classmethod
    def get_all_lexical_forms(cls):
        if LexicalForm.all_lexical_forms is None:
            smp_service = SmpWebService()
            lexical_forms_json = smp_service.get_lexical_forms()
            lexical_forms = LexicalForm.get_lexical_forms_from_json(lexical_forms_json)
            LexicalForm.all_lexical_forms = lexical_forms
        return LexicalForm.all_lexical_forms


    # class method get lexical form from json
    @classmethod
    def get_lexical_form_from_json(cls, lexical_form_json):
        lexical_form = LexicalForm()
        lexical_form.form_type = lexical_form_json['form_type']
        lexical_form.language = Language.objects.get(id=lexical_form_json['language'])
        lexical_form.lexical_name_id = lexical_form_json['lexical_name']
        lexical_form.is_in_book = lexical_form_json['is_in_book']
        lexical_form.name = lexical_form_json['lexical_name']
        lexical_form.id = lexical_form_json['id']
        lexical_form.pk = lexical_form_json['id']
        return lexical_form

    @classmethod
    def get_lexical_forms_from_json(cls, lexical_forms_json):
        lexical_forms = []
        languages = Language.objects.all()
        lexical_names = LexicalName.objects.all()
        for lexical_form_json in lexical_forms_json:
            lexical_form = LexicalForm()
            lexical_form.form_type = lexical_form_json['form_type']
            lexical_form.language = languages.get(id=lexical_form_json['language'])
            lexical_form.lexical_name = lexical_names.get(id=lexical_form_json['lexical_name'])
            lexical_form.is_in_book = lexical_form_json['is_in_book']
            lexical_form.name = lexical_form_json['name']
            lexical_form.id = lexical_form_json['id']
            lexical_form.pk = lexical_form_json['id']
            lexical_forms.append(lexical_form)
        return lexical_forms

    @classmethod
    def create(cls, lexical_form):
        smp_service = SmpWebService()
        lexical_form_json = smp_service.create_lexical_form(lexical_form, lexical_form.author.token, lexical_form.csrftoken)
        new_lexical_form = LexicalForm.get_lexical_form_from_json(lexical_form_json)
        LexicalForm.all_lexical_forms = None
        return new_lexical_form

    def save(self, commit=True):
        smp_service = SmpWebService()
        lexical_form_json = smp_service.update_lexical_form(self, self.author.token, self.csrftoken)
        lexical_form = LexicalForm.get_lexical_form_from_json(lexical_form_json)
        LexicalForm.all_lexical_forms = None
        return lexical_form

    #delete lexical form
    def delete(self):
        smp_service = SmpWebService()
        ok = smp_service.delete_lexical_form(self, self.author.token, self.csrftoken)
        LexicalForm.all_lexical_forms = None
        return ok

    #validate unique lexical form
    def validate_unique(self, exclude):
        lexical_forms = LexicalForm.objects.all()
        for lexical_form in lexical_forms:
            if (lexical_form.form_type == self.form_type and lexical_form.language == self.language
                    and lexical_form.lexical_name == self.lexical_name and lexical_form.name == self.name):
                #check that id is not the same
                if (hasattr(self, 'id')) or (self.id is not None):
                    if lexical_form.id != self.id:
                        raise ValidationError(
                        ('Lexikal form med detta namn finns redan för detta lexikala namn.'),
                        params={'form_type': self.form_type, 'language': self.language, 'lexical_name': self.lexical_name, 'name': self.name}
                        )
                else:
                    raise ValidationError(
                        ('Kunde inte spara lexikal form.'),
                    )

