from django.core.exceptions import ValidationError
from django.urls import reverse

from ISOF.model.simple_model_query_set import SimpleModelQuerySet
from ISOF.model.web_service.smp_web_service import SmpWebService
from django.db import models


class LexicalNameQuerySet(SimpleModelQuerySet):
    def _fetch_all(self):
        if (not hasattr(self, '_result_cache')) or (self._result_cache is None):
            self._result_cache = LexicalName.get_all_lexical_names()
            self.create_dictionary(self._result_cache)

    def get(self, *args, **kwargs):
        """
        Perform the query and return a single object matching the given
        keyword arguments.
        """
        self._fetch_all()
        obj_id = None

        if(kwargs):
            if 'id' in kwargs:
                obj_id = kwargs['id']
            if 'pk' in kwargs:
                obj_id = kwargs['pk']

        if args:
            obj_id = args[0].children[0][1]


        if isinstance(obj_id, str):
            obj_id = int(obj_id)
        if self.has_dictionary():
            return self._result_cache_dictionary[obj_id]
        else:
            for obj in self._result_cache:
                if (obj.id == obj_id):
                    return obj

        return self

    def filter(self, *args, **kwargs):
        if (len(args) > 0):
            search_string = args[0].children[0].children[0][1]
            if (search_string):
                if (search_string.startswith("efterled:")):
                    search_string = search_string[9:]
                    lexical_names = LexicalName.get_all_lexical_names()
                    lexical_names = [lexical_name
                                     for lexical_name in lexical_names
                                     if lexical_name.last_element and
                                     lexical_name.last_element.lower().startswith(search_string.lower())]

                    queryset = LexicalNameQuerySet(LexicalName)
                    queryset._result_cache = lexical_names
                    queryset.create_dictionary(queryset._result_cache)
                else:
                    lexical_names = LexicalName.get_lexical_names_by_search_string(self, search_string)
                    queryset = LexicalNameQuerySet(LexicalName)
                    queryset._result_cache = lexical_names
                    queryset.create_dictionary(queryset._result_cache)
                return queryset

        if (len(kwargs) > 0) and isinstance(kwargs, dict) and ('id__in' in kwargs):
            id_set = kwargs['id__in']
            lexical_names = []
            if len(id_set) > 0:
                lexical_name_id = int(list(id_set)[0])
                if self.has_dictionary():
                    lexical_names.append(self._result_cache_dictionary[lexical_name_id])
                else:
                    for obj in self._result_cache:
                        if obj.id == lexical_name_id:
                            lexical_names.append(obj)
                            break

            queryset = LexicalNameQuerySet(LexicalName)
            queryset._result_cache = lexical_names
            return queryset

        return self


class LexicalNameManager(models.Manager):
    def get_queryset(self):
        queryset = LexicalNameQuerySet(LexicalName)
        return queryset


class LexicalName(models.Model):
    all_lexical_names = None

    SEX = (
        ('Man', 'Man'),
        ('Woman', 'Kvinna'),
        ('Unknown', 'Okänt')
    )
    NAME_TYPE = (
        ('Surname', 'Tillnamn'),
        ('FirstName', 'Förnamn')
    )

    LEXICAL_NAME_TYPE = (
        ('Unknown', 'Okänt'),
        ('Composed', 'Sammansatt'),
        ('NotComposed', 'Ej sammansatt'),
        ('VariedName', 'Variationsnamn'),
        ('Prelude', 'Förled'),
    )

    ORIGIN = (
        ('Nordic', 'Nordiskt'),
        ('Borrowed', 'Inlånat'),
        ('Uncertain', 'Osäkert'),
        ('Finnish', 'Finskt')
    )

    name = models.CharField(null=False, blank=False, max_length=256, verbose_name="Namn")
    name_uncertain = models.BooleanField(default=False, null=True, verbose_name="Osäkert namn")
    first_element = models.CharField(max_length=256, null=False, default='', blank=True, verbose_name="Förled")
    first_element_uncertain = models.BooleanField(default=False, null=True, verbose_name="Osäkert förled")
    last_element = models.CharField(max_length=256, null=False, default='', blank=True, verbose_name="Efterled")
    last_element_uncertain = models.BooleanField(default=False, null=True, verbose_name="Osäkert efterled")
    sex = models.CharField(null=False, blank=False, choices=SEX, max_length=10, verbose_name="Kön")
    is_sex_certain = models.BooleanField(default=False, null=True, verbose_name="Osäkert")
    lexical_name_type = models.CharField(null=False, blank=False, choices=LEXICAL_NAME_TYPE, max_length=20,
                                         verbose_name="Namnbildningstyp")
    lexical_name_type_uncertain = models.BooleanField(default=False, null=True, verbose_name="Osäker")
    name_type = models.CharField(null=False, blank=False, max_length=20, choices=NAME_TYPE,
                                 verbose_name="Namntyp")
    text_in_article = models.TextField(max_length=10000, null=True, blank=True, verbose_name="Namnkommentar")
    origin = models.CharField(null=False, blank=False, max_length=40, verbose_name="Ursprung", choices=ORIGIN)
    uncertain_origin = models.BooleanField(default=False, null=True, verbose_name="Osäkert")
    detailed_origin = models.CharField(null=True, blank=True, max_length=50, verbose_name="Detaljerat ursprung")
    is_in_book = models.BooleanField(default=True, null=True, verbose_name="Ingår i boken")
    frequency_in_book = models.IntegerField(null=True, blank=True, verbose_name="Antal belägg")
    internal_note = models.TextField(max_length=4000, null=True, blank=True, verbose_name="Intern kommentar")
    external_note = models.TextField(max_length=4000, null=True, blank=True, verbose_name="Extern kommentar")

    pdf_file_link = models.CharField(max_length=2000, null=True, blank=True, verbose_name="PDF-länk")
    name_form_comment = models.TextField(max_length=10000, null=True, blank=True, verbose_name="Namnformskommentar")
    is_public = models.BooleanField(default=False, null=True, verbose_name="Är publik")
    has_only_compound_names = models.BooleanField(default=False, null=False, verbose_name="Endast i sammansättning")

    change_date = models.DateTimeField(null=True, blank=True, auto_now=True, verbose_name="Ändringsdatum")

    # str method
    objects = LexicalNameManager()

    #create meta class for LexicalName
    class Meta:
        base_manager_name = 'objects'
        managed = True
        verbose_name = "lexikala namn"
        verbose_name_plural = "lexikala namn"

    def __str__(self):
        if (self.name_type):
            return self.name + ': ' + dict(LexicalName.NAME_TYPE)[self.name_type]
        else:
            return self.name

    def __hash__(self):
        return hash((self.id, self.lexical_name_type))


    @classmethod
    def get_all_lexical_names(cls):
        if LexicalName.all_lexical_names is None:
            smp_service = SmpWebService()
            lexical_names_json = smp_service.get_lexical_names()
            LexicalName.all_lexical_names = LexicalName.get_lexical_names_from_json(lexical_names_json)
        return LexicalName.all_lexical_names

    #class method get lexical names from search string
    def get_lexical_names_by_search_string(self, search_string):
        smp_service = SmpWebService()
        lexical_names_json = smp_service.get_lexical_names_by_search_string(search_string)
        lexical_names = LexicalName.get_lexical_names_from_json(lexical_names_json)
        return lexical_names


    @classmethod
    def get_lexical_name_from_json(cls, lexical_name_json):
        lexical_name = LexicalName()
        lexical_name.lexical_name_type = lexical_name_json['lexical_name_type']
        lexical_name.last_element = lexical_name_json['last_element']
        lexical_name.is_in_book = lexical_name_json['is_in_book']
        lexical_name.text_in_article = lexical_name_json['text_in_article']
        lexical_name.origin = lexical_name_json['origin']
        lexical_name.detailed_origin = lexical_name_json['detailed_origin']
        lexical_name.name_type = lexical_name_json['name_type']
        lexical_name.name = lexical_name_json['name']
        lexical_name.sex = lexical_name_json['sex']
        lexical_name.internal_note = lexical_name_json['internal_note']
        lexical_name.external_note = lexical_name_json['external_note']
        lexical_name.pdf_file_link = lexical_name_json['pdf_file_link']
        lexical_name.change_date = lexical_name_json['change_date']
        lexical_name.id = lexical_name_json['id']
        lexical_name.pk = lexical_name_json['id']
        lexical_name.is_sex_certain =  lexical_name_json['is_sex_certain']
        lexical_name.uncertain_origin = lexical_name_json['uncertain_origin']
        lexical_name.name_uncertain = lexical_name_json['name_uncertain']
        lexical_name.last_element_uncertain = lexical_name_json['last_element_uncertain']
        lexical_name.lexical_name_type_uncertain = lexical_name_json['lexical_name_type_uncertain']
        lexical_name.frequency_in_book = lexical_name_json['frequency_in_book']
        lexical_name.name_form_comment = lexical_name_json['name_form_comment']
        lexical_name.is_public = lexical_name_json['is_public']
        lexical_name.has_only_compound_names = lexical_name_json['has_only_compound_names']
        lexical_name.first_element = lexical_name_json['first_element']
        lexical_name.first_element_uncertain = lexical_name_json['first_element_uncertain']
        return lexical_name


    @classmethod
    def get_lexical_names_from_json(cls, lexical_names_json):
        lexical_names = []
        for lexical_name_json in lexical_names_json:
            lexical_name = LexicalName()
            lexical_name.name = lexical_name_json['name']
            lexical_name.sex = lexical_name_json['sex']
            lexical_name.lexical_name_type = lexical_name_json['lexical_name_type']
            lexical_name.last_element = lexical_name_json['last_element']
            lexical_name.is_in_book = lexical_name_json['is_in_book']
            lexical_name.text_in_article = lexical_name_json['text_in_article']
            lexical_name.origin = lexical_name_json['origin']
            lexical_name.detailed_origin = lexical_name_json['detailed_origin']
            lexical_name.name_type = lexical_name_json['name_type']
            lexical_name.internal_note = lexical_name_json['internal_note']
            lexical_name.external_note = lexical_name_json['external_note']
            lexical_name.pdf_file_link = lexical_name_json['pdf_file_link']
            lexical_name.change_date = lexical_name_json['change_date']
            lexical_name.id = lexical_name_json['id']
            lexical_name.pk = lexical_name_json['id']
            lexical_name.is_sex_certain = lexical_name_json['is_sex_certain']
            lexical_name.uncertain_origin = lexical_name_json['uncertain_origin']
            lexical_name.name_uncertain = lexical_name_json['name_uncertain']
            lexical_name.last_element_uncertain = lexical_name_json['last_element_uncertain']
            lexical_name.lexical_name_type_uncertain = lexical_name_json['lexical_name_type_uncertain']
            lexical_name.frequency_in_book = lexical_name_json['frequency_in_book']
            lexical_name.name_form_comment = lexical_name_json['name_form_comment']
            lexical_name.is_public = lexical_name_json['is_public']
            lexical_name.has_only_compound_names = lexical_name_json['has_only_compound_names']
            lexical_name.first_element = lexical_name_json['first_element']
            lexical_name.first_element_uncertain = lexical_name_json['first_element_uncertain']
            lexical_names.append(lexical_name)

        return lexical_names


    # create lexical name
    @classmethod
    def create(cls, lexical_name):
        smp_service = SmpWebService()
        lexical_name_json = smp_service.create_lexical_name(lexical_name, lexical_name.author.token, lexical_name.csrftoken)
        new_lexical_name = LexicalName.get_lexical_name_from_json(lexical_name_json)
        LexicalName.all_lexical_names = None
        return new_lexical_name

    # delete regular method
    def delete(self):
        smp_service = SmpWebService()
        smp_service.delete_lexical_name(self, self.author.token, self.csrftoken)
        LexicalName.all_lexical_names = None


    # save with commit true
    def save(self, commit=True):
        smp_service = SmpWebService()
        smp_service.update_lexical_name(self, self.author.token, self.csrftoken)
        LexicalName.all_lexical_names = None
        return self

    #generate search form for lexical name
    @classmethod
    def generate_search_form(cls, lexical_form):
        #call smp service
        smp_service = SmpWebService()
        search_form_json = smp_service.generate_search_form_for_lexical_name(lexical_form.lexical_name_id, lexical_form.author.token, lexical_form.csrftoken)
        return search_form_json


    # validate unique
    def validate_unique(self, exclude):
        lexical_names = LexicalName.objects.all()
        for lexical_name in lexical_names:
            if (lexical_name.lexical_name_type == self.lexical_name_type
                    and lexical_name.name == self.name
                    and lexical_name.last_element == self.last_element):

                if (hasattr(self, 'id')) or (self.id is not None):
                    if lexical_name.id != self.id:
                        raise ValidationError(
                            {'lexical_name_type': ["Lexikalt namn med samma namn och efterled och namnbildningstyp finns redan."]})
                else:
                    raise ValidationError(
                        {'lexical_name_type': [
                            "Kunde inte spara lexikalt namn."]})
