from django.core.exceptions import ValidationError
from django.db import models
from ISOF.model.simple_model_query_set import SimpleModelQuerySet
from ISOF.model.web_service.smp_web_service import SmpWebService


class LocalityRelationQuerySet(SimpleModelQuerySet):
    def _fetch_all(self):
        if (not hasattr(self, '_result_cache')) or (self._result_cache is None):
            self._result_cache = LocalityRelation.get_all_locality_relations()
            self.create_dictionary(self._result_cache)

    def filter(self, *args, **kwargs):
        """
        Return a new QuerySet instance with the args ANDed to the existing
        set.
        """
        if len(args) > 0:
            search_string = args[0].children[0].children[0][1]
            if search_string:
                locality_relations = []
                self._fetch_all()
                if self._result_cache is not None:
                    for locality_relation in self._result_cache:
                        if (search_string.casefold() in locality_relation.locality_relation.casefold()):
                            locality_relations.append(locality_relation)

                queryset = LocalityRelationQuerySet(LocalityRelation)
                queryset._result_cache = locality_relations
                return queryset

        return self


class LocalityRelationManager(models.Manager):
    def get_queryset(self):
        queryset = LocalityRelationQuerySet(LocalityRelation)
        return queryset


class LocalityRelation(models.Model):
    article_text = models.CharField(max_length=250, null=True, blank=True,
                                    unique=False, verbose_name="Text i artiklar")
    internal_comment = models.CharField(max_length=250, null=True, blank=True,
                                        unique=False, verbose_name="Intern kommentar")
    locality_relation = models.CharField(max_length=250, null=False, blank=False,
                                         unique=True, verbose_name="Lokaliseringsrelation")
    objects = LocalityRelationManager()
    all_locality_relations = None

    class Meta:
        base_manager_name = 'objects'
        managed = True
        verbose_name = "lokaliseringsrelation"
        verbose_name_plural = "lokaliseringsrelationer"

    def __str__(self):
        return self.locality_relation

    @classmethod
    def create(cls, locality_relation):
        smp_service = SmpWebService()
        locality_relation_json = smp_service.create_locality_relation(locality_relation,
                                                                      locality_relation.author.token,
                                                                      locality_relation.csrftoken)
        LocalityRelation.all_locality_relations = None
        return LocalityRelation.get_locality_relation_from_json(locality_relation_json)

    def delete(self):
        smp_service = SmpWebService()
        LocalityRelation.all_locality_relations = None
        ok = smp_service.delete_locality_relation(self, self.author.token, self.csrftoken)
        return ok

    # Get all locality_relations and set in LocalityRelation.all_locality_relations
    @classmethod
    def get_all_locality_relations(cls):
        if LocalityRelation.all_locality_relations is None:
            smp_service = SmpWebService()
            locality_relations_json = smp_service.get_locality_relations()
            locality_relations = []
            for locality_relation_json in locality_relations_json:
                locality_relations.append(LocalityRelation.get_locality_relation_from_json(locality_relation_json))
            LocalityRelation.all_locality_relations = locality_relations
        return LocalityRelation.all_locality_relations

    @classmethod
    def get_locality_relation_from_json(cls, locality_relation_json):
        locality_relation = LocalityRelation()
        locality_relation.article_text = locality_relation_json['article_text']
        locality_relation.internal_comment = locality_relation_json['internal_comment']
        locality_relation.locality_relation = locality_relation_json['locality_relation']
        locality_relation.id = locality_relation_json['id']
        locality_relation.pk = locality_relation_json['id']
        return locality_relation

    def save(self, commit=True):
        smp_service = SmpWebService()
        smp_service.update_locality_relation(self, self.author.token, self.csrftoken)
        LocalityRelation.all_locality_relations = None
        return self

    def validate_unique(self, exclude):
        """
        Check that unique fields are unique for this data type.
        """

        # Check that abbreviation is unique.
        locality_relations = LocalityRelation.objects.all()
        for locality_relation in locality_relations:
            # Check that abbreviation is unique.
            if self.locality_relation == locality_relation.locality_relation:
                # Check that id is not None.
                if (hasattr(self, 'id')) or (self.id is None):
                    if self.id != locality_relation.id:
                        raise ValidationError(f'Det finns redan en lokaliseringsrelation "{self.locality_relation}".')
                else:
                    raise ValidationError(f'Det finns redan en lokaliseringsrelation "{self.locality_relation}".')
