from django.core.exceptions import ValidationError
from django.db import models
from ISOF.model.simple_model_query_set import SimpleModelQuerySet
from ISOF.model.web_service.smp_web_service import SmpWebService


class NameLastElementQuerySet(SimpleModelQuerySet):
    def _fetch_all(self):
        if (not hasattr(self, '_result_cache')) or (self._result_cache is None):
            smp_service = SmpWebService()
            name_last_elements_json = smp_service.get_name_last_elements()
            name_last_elements = NameLastElement.get_name_last_elements_from_json(name_last_elements_json)
            self._result_cache = name_last_elements

    def filter(self, *args, **kwargs):
        """
        Return a new QuerySet instance with the args ANDed to the existing
        set.
        """
        if (len(args) > 0):
            search_string = args[0].children[0].children[0][1]
            if (search_string):
                name_last_elements = []
                self._fetch_all()
                if (self._result_cache is not None):
                    for name_last_element in self._result_cache:
                        if (search_string in name_last_element.last_element):
                            name_last_elements.append(name_last_element)
            queryset = NameLastElementQuerySet(NameLastElement)
            queryset._result_cache = name_last_elements
            return queryset

        return self

    # Create NameLastElement manager


class NameLastElementManager(models.Manager):
    def get_queryset(self):
        queryset = NameLastElementQuerySet(NameLastElement)
        return queryset


# Create model for NameLastElement
class NameLastElement(models.Model):
    # Create fields for NameLastElement
    last_element = models.CharField(max_length=256, null=False, blank=False, verbose_name="Efterled")
    sort_order = models.IntegerField(null=False, blank=False, default=0, verbose_name="Sorteringsordning")
    objects = NameLastElementManager()

    # Create meta class for NameLastElement
    class Meta:
        base_manager_name = 'objects'
        managed = True
        verbose_name = "Efterled"
        verbose_name_plural = "Efterled"
        ordering = ['sort_order']

    # Create string method for NameLastElement
    def __str__(self):
        if (hasattr(self, 'id')) and (self.id is not None):
            return f'Efterled: {self.last_element} (id={self.id})'
        else:
            return f'Efterled: {self.last_element}'

    #equals method

     # Create get_name_last_element_from_json method
    @classmethod
    def get_name_last_element_from_json(cls, name_last_element_json):
        name_last_element = NameLastElement()
        name_last_element.last_element = name_last_element_json['last_element']
        name_last_element.sort_order = name_last_element_json['sort_order']
        name_last_element.id = name_last_element_json['id']
        name_last_element.pk = name_last_element_json['id']
        return name_last_element

    # Create get_name_last_elements_from_json method
    @classmethod
    def get_name_last_elements_from_json(cls, name_last_elements_json):
        name_last_elements = []
        for name_last_element_json in name_last_elements_json:
            name_last_element = NameLastElement()
            name_last_element.last_element = name_last_element_json['last_element']
            name_last_element.sort_order = name_last_element_json['sort_order']
            name_last_element.id = name_last_element_json['id']
            name_last_element.pk = name_last_element_json['id']
            name_last_elements.append(name_last_element)
        return name_last_elements

    # Class method create
    @classmethod
    def create(cls, last_element):
        # Create name_last_element in SmpService.
        smp_service = SmpWebService()
        name_last_element_json = smp_service.create_name_last_element(last_element, last_element.author.token,
                                                                      last_element.csrftoken)
        name_last_element = NameLastElement.get_name_last_element_from_json(name_last_element_json)
        return name_last_element


    def delete(self):
        # Delete name_last_element in SmpService.
        smp_service = SmpWebService()
        ok = smp_service.delete_name_last_element(self, self.author.token, self.csrftoken)
        return ok

    def save(self, commit=True):
        # Save name last name in SmpService.
        smp_service = SmpWebService()
        smp_service.update_name_last_element(self, self.author.token, self.csrftoken)
        return self

    def validate_unique(self, exclude):
        """
        Check that unique fields are unique for this data type.
        """

        # Check that last_element is unique.
        name_last_elements = NameLastElement.objects.all()
        for name_last_element in name_last_elements:
            if (self.last_element == name_last_element.last_element):
                if (hasattr(self, 'id')) or (self.id is None):
                    if (self.id != name_last_element.id):
                        raise ValidationError(f'Det finns redan ett efterled "{self.last_element}".')
                else:
                    raise ValidationError(f'Det finns redan ett efterled "{self.last_element}".')
