import datetime
from time import strptime

from django.db import models

from ISOF.model.simple_model_query_set import SimpleModelQuerySet
from ISOF.model.web_service.smp_web_service import SmpWebService


class PersonQuerySet(SimpleModelQuerySet):
    def _fetch_all(self):
        if (not hasattr(self, '_result_cache')) or (self._result_cache is None):
            self._result_cache = Person.get_all_persons()
            self.create_dictionary(self._result_cache)

    def filter(self, *args, **kwargs):
        if (len(args) > 0):
            filter_tuple = args[0].children[0]
            if hasattr(filter_tuple, 'children'):
                search_string = filter_tuple.children[0][1].casefold()
                if (search_string):
                    persons = []
                    self._fetch_all()
                    if (self._result_cache is not None):
                        for person in self._result_cache:
                            if (search_string in person.name.casefold()):
                                persons.append(person)
                    queryset = PersonQuerySet(Person)
                    queryset._result_cache = persons
                    queryset.create_dictionary(queryset._result_cache)
                    return queryset

        if (len(kwargs) > 0) and isinstance(kwargs, dict) and ('id__in' in kwargs):
            id_set = kwargs['id__in']
            persons = []
            if len(id_set) > 0:
                person_id = int(list(id_set)[0])
                if self.has_dictionary():
                    persons.append(self._result_cache_dictionary[person_id])
                else:
                    for obj in self._result_cache:
                        if obj.id == person_id:
                            person_id.append(obj)
                            break

            queryset = PersonQuerySet(Person)
            queryset._result_cache = persons
            return queryset

        else:
            return self


class PersonManager(models.Manager):
    def get_queryset(self):
        queryset = PersonQuerySet(Person)
        return queryset

#person form to use in person admin


class Person(models.Model):
    all_persons = None
    date_of_birth = models.DateField(null=True, blank=True, default=None, verbose_name="Födelsedatum")
    date_of_death = models.DateField(null=True, blank=True, default=None, verbose_name="Dödsdatum")
    external_note = models.TextField(max_length=4000, null=True, blank=True, verbose_name="Extern kommentar")
    index = models.IntegerField(null=True, blank=True, verbose_name="Personnamns index")
    internal_note = models.TextField(max_length=4000, null=True, blank=True, verbose_name="Intern kommentar")
    is_in_book = models.BooleanField(default=False, null=False, verbose_name="Finns i boken")
    name = models.CharField(null=False, blank=False, max_length=256, verbose_name="Namn")

    objects = PersonManager()

    class Meta:
        base_manager_name = 'objects'
        verbose_name = "Person"
        verbose_name_plural = "Personer"
        ordering = ['name']

    def __str__(self):
        return self.name

    @staticmethod
    def get_all_persons():
        if Person.all_persons is None:
            smp_service = SmpWebService()
            persons_json = smp_service.get_persons()
            Person.all_persons = Person.get_all_persons_from_json(persons_json)
        return Person.all_persons

    #get person by id
    def get_person_by_id(id):
        persons = Person.get_all_persons()
        for person in persons:
            if person.id == id:
                return person
        return None

    @staticmethod
    def get_all_persons_from_json(persons_json):
        persons = []
        for person_json in persons_json:
            person = Person.get_person_from_json(person_json)
            persons.append(person)
        return persons

    @staticmethod
    def get_person_from_json(person_json):
        person = Person()
        person.id = person_json['id']
        person.external_note = person_json['external_note']
        person.internal_note = person_json['internal_note']
        person.is_in_book = person_json['is_in_book']
        person.name = person_json['name']
        person.index = person_json['index']
        #convert date strings to django datetime objects
        if (person_json['date_of_birth'] is not None):
            person.date_of_birth = datetime.datetime.strptime(person_json['date_of_birth'], '%Y-%m-%d').date()
        if (person_json['date_of_death'] is not None):
           person.date_of_death = datetime.datetime.strptime(person_json['date_of_death'], '%Y-%m-%d').date()

        return person

    @staticmethod
    def create(cls, person):
        smp_service = SmpWebService()
        person_json = smp_service.create_person(person, person.author.token, person.csrftoken)
        person = Person.get_person_from_json(person_json)
        Person.all_persons = None
        return person

    #save
    def save(self, commit=True):
        smp_service = SmpWebService()
        person_json = smp_service.update_person(self, self.author.token, self.csrftoken)
        person = Person.get_person_from_json(person_json)
        Person.all_persons = None
        return person


    # delete
    def delete(self):
        smp_service = SmpWebService()
        smp_service.delete_person(self, self.author.token, self.csrftoken)
        Person.all_persons = None





