from ISOF.model.web_service.place_name_web_service import PlaceNameWebService
import threading
from ISOF.common.string_functions import is_empty_string, is_integer_string
from ISOF.model.place_name_search_criteria import PlaceNameSearchCriteria
from ISOF.model.place_name_search_result import PlaceNameSearchResult


class PlaceNameManager():
    country_area_filters = None
    language_filters = None
    lock = threading.Lock()
    name_types = None
    place_name_basis_types = None

    def __init__(self):
        self.data_source = PlaceNameWebService()

    def get_country_area_filter(self, country_area_filter_id):
        for country_area_filter in self.get_country_area_filters():
            if (country_area_filter_id == country_area_filter.id):
                return country_area_filter

    def get_country_area_filters(self):
        self.__load_data()
        return PlaceNameManager.country_area_filters

    def get_language_filter(self, language_filter_code_3):
        for language_filter in self.get_language_filters():
            if (language_filter_code_3 == language_filter.code_3):
                return language_filter

    def get_language_filters(self):
        self.__load_data()
        return PlaceNameManager.language_filters

    def get_name_type(self, name_type_id):
        for name_type in self.get_name_types():
            if (name_type_id == name_type.id):
                return name_type

    def get_name_types(self):
        self.__load_data()
        return PlaceNameManager.name_types

    def get_place_name(self, place_name_id):
        return self.data_source.get_place_name(place_name_id)

    def get_place_name_basis(self, place_name_basis_search_criteria=None):
        return self.data_source.get_place_name_basis(place_name_basis_search_criteria)

    def get_place_name_basis_back_side_image(self, place_name_basis_id):
        return self.data_source.get_place_name_basis_back_side_image(place_name_basis_id)

    def get_place_name_basis_by_id(self, place_name_basis_id):
        return self.data_source.get_place_name_basis_by_id(place_name_basis_id)

    def get_place_name_basis_front_side_image(self, place_name_basis_id):
        return self.data_source.get_place_name_basis_front_side_image(place_name_basis_id)

    def get_place_name_basis_pdf(self, place_name_basis_id):
        return self.data_source.get_place_name_basis_pdf(place_name_basis_id)

    def get_place_name_basis_type(self, place_name_basis_type_id):
        for place_name_basis_type in self.get_place_name_basis_types():
            if (place_name_basis_type.id == place_name_basis_type_id):
                return place_name_basis_type
        return None

    def get_place_name_basis_types(self):
        self.__load_data()
        return PlaceNameManager.place_name_basis_types

    def get_place_name_search_criteria_country_areas(self, place_name_search_criteria):
        country_areas = ''

        if (hasattr(place_name_search_criteria, 'country_area_filter_ids')):
            if (isinstance(place_name_search_criteria.country_area_filter_ids, int)):
                country_area = self.get_country_area_filter(place_name_search_criteria.country_area_filter_ids)
                if (country_areas != ''):
                    country_areas = country_areas + ', '
                country_areas = country_areas + country_area.name

            if (isinstance(place_name_search_criteria.country_area_filter_ids, list)):
                for country_area_filter_id in place_name_search_criteria.country_area_filter_ids:
                    country_area = self.get_country_area_filter(country_area_filter_id)
                    if (country_areas != ''):
                        country_areas = country_areas + ', '
                    country_areas = country_areas + country_area.name

        return country_areas

    def get_place_name_search_criteria_languages(self, place_name_search_criteria):
        languages = ''

        if (hasattr(place_name_search_criteria, 'language_filter_codes_3')):
            if (isinstance(place_name_search_criteria.language_filter_codes_3, str)):
                language = self.get_language_filter(place_name_search_criteria.language_filter_codes_3)
                if (languages != ''):
                    languages = languages + ', '
                languages = languages + language.name

            if (isinstance(place_name_search_criteria.language_filter_codes_3, list)):
                for language_filter_code_3 in place_name_search_criteria.language_filter_codes_3:
                    language = self.get_language_filter(language_filter_code_3)
                    if (languages != ''):
                        languages = languages + ', '
                    languages = languages + language.name

        return languages

    def get_place_name_search_criteria_name_types(self, place_name_search_criteria):
        name_types = ''

        if (hasattr(place_name_search_criteria, 'name_type_ids')):
            if (isinstance(place_name_search_criteria.name_type_ids, int)):
                name_type = self.get_name_type(place_name_search_criteria.name_type_ids)
                if (name_types != ''):
                    name_types = name_types + ', '
                name_types = name_types + name_type.name

            if (isinstance(place_name_search_criteria.name_type_ids, list)):
                for name_type_id in place_name_search_criteria.name_type_ids:
                    name_type = self.get_name_type(name_type_id)
                    if (name_types != ''):
                        name_types = name_types + ', '
                    name_types = name_types + name_type.name

        return name_types

    def get_place_name_search_criteria_place_name_basis_types(self, place_name_search_criteria):
        place_name_basis_types = ''

        if (hasattr(place_name_search_criteria, 'place_name_basis_type_ids')):
            if (isinstance(place_name_search_criteria.place_name_basis_type_ids, int)):
                place_name_basis_type = self.get_place_name_basis_type(place_name_search_criteria.place_name_basis_type_ids)
                if (place_name_basis_types != ''):
                    place_name_basis_types = place_name_basis_types + ', '
                place_name_basis_types = place_name_basis_types + place_name_basis_type.name

            if (isinstance(place_name_search_criteria.place_name_basis_type_ids, list)):
                for place_name_basis_type_id in place_name_search_criteria.place_name_basis_type_ids:
                    place_name_basis_type = self.get_place_name_basis_type(place_name_basis_type_id)
                    if (place_name_basis_types != ''):
                        place_name_basis_types = place_name_basis_types + ', '
                    place_name_basis_types = place_name_basis_types + place_name_basis_type.name

        return place_name_basis_types

    def get_place_names(self,
                        place_name_search_criteria=None,
                        page_number=None,
                        page_size=None,
                        place_name_sort_field=None,
                        place_name_sort_order=None):
        if ((place_name_search_criteria is not None) and
            (place_name_search_criteria.place_name_search_type == 'PlaceNameId')):
            # Get place name by id.
            place_name_search_result = PlaceNameSearchResult()

            # Check for valid place name id.
            if (is_empty_string(place_name_search_criteria.place_name_search_string)):
                place_name_search_result.message = 'NoPlaceNameIdProvided'
                return place_name_search_result
            if (not is_integer_string(place_name_search_criteria.place_name_search_string)):
                place_name_search_result.message = 'InvalidPlaceNameIdProvided'
                return place_name_search_result

            # Get place name.
            place_name_id = int(place_name_search_criteria.place_name_search_string)
            place_name = self.get_place_name(place_name_id)
            if (place_name is None):
                place_name_search_result.message = 'NoPlaceNameWithProvidedIdFound'
                return place_name_search_result

            place_name_search_result.count = 1
            place_name_search_result.message = 'PlaceNamesFound'
            place_name_search_result.page_number = 1
            if (page_size is None):
                place_name_search_result.page_size = 20
            else:
                place_name_search_result.page_size = page_size
            place_name_search_result.place_names = [place_name]

            return place_name_search_result

        if ((place_name_search_criteria is not None) and
            (place_name_search_criteria.place_name_search_type == 'PlaceNameBasisId')):
            # Get place name by place name basis id.
            place_name_search_result = PlaceNameSearchResult()

            # Check for valid place name basis id.
            if (is_empty_string(place_name_search_criteria.place_name_search_string)):
                place_name_search_result.message = 'NoPlaceNameBasisIdProvided'
                return place_name_search_result
            if (not is_integer_string(place_name_search_criteria.place_name_search_string)):
                place_name_search_result.message = 'InvalidPlaceNameBasisIdProvided'
                return place_name_search_result

            # Get place name basis.
            place_name_basis_id = int(place_name_search_criteria.place_name_search_string)
            place_name_basis = self.get_place_name_basis_by_id(place_name_basis_id)
            if (place_name_basis is None):
                place_name_search_result.message = 'NoPlaceNameBasisWithProvidedIdFound'
                return place_name_search_result

            # Get place name.
            place_name = self.get_place_name(place_name_basis.place_name_id)
            if (place_name is None):
                place_name_search_result.message = 'NoPlaceNameWithProvidedIdFound'
                return place_name_search_result

            place_name_search_result.count = 1
            place_name_search_result.message = 'PlaceNamesFound'
            place_name_search_result.page_number = 1
            if (page_size is None):
                place_name_search_result.page_size = 20
            else:
                place_name_search_result.page_size = page_size
            place_name_search_result.place_names = [place_name]

            return place_name_search_result

        # Get place names by search criteria.
        place_name_search_result = self.data_source.get_place_names(place_name_search_criteria,
                                                                    page_number,
                                                                    page_size,
                                                                    place_name_sort_field,
                                                                    place_name_sort_order)
        if (place_name_search_result.count == 0):
            place_name_search_result.message = 'NoPlaceNamesFound'
        else:
            place_name_search_result.message = 'PlaceNamesFound'
        return place_name_search_result

    def __load_data(self):
        if ((PlaceNameManager.country_area_filters is None) or
            (PlaceNameManager.language_filters is None) or
            (PlaceNameManager.name_types is None) or
            (PlaceNameManager.place_name_basis_types is None)):

            PlaceNameManager.lock.acquire()
            try:
                if (PlaceNameManager.country_area_filters is None):
                    PlaceNameManager.country_area_filters = self.data_source.get_country_area_filters()
                if (PlaceNameManager.language_filters is None):
                    PlaceNameManager.language_filters = self.data_source.get_language_filters()
                if (PlaceNameManager.name_types is None):
                    PlaceNameManager.name_types = self.data_source.get_name_types()
                if (PlaceNameManager.place_name_basis_types is None):
                    PlaceNameManager.place_name_basis_types = self.data_source.get_place_name_basis_types()

            finally:
                PlaceNameManager.lock.release()
