from ISOF.common.string_functions import is_not_empty_string

class PlaceNameSearchCriteria():

    PLACE_NAME_SEARCH_TYPE = (
        ('BeginsWith', 'Börjar med'),
        ('Contains', 'Innehåller'),
        ('EndsWith', 'Slutar med'),
        ('Similar', 'Liknande'),
        ('Equal', 'Exakt'),
        ('Wildcard', 'Jokertecken'),
        ('PlaceNameId', 'Id för ortnamn'),
        ('PlaceNameBasisId', 'Id för arkivpost'),
    )

    # country_area_filter_ids = List[models.IntegerField(null=True))]
    # country_area_names = List[models.CharField(max_length=256, null=True, blank=True)]
    # county_id = models.IntegerField(null=True)
    # district_id = models.IntegerField(null=True)
    # language_filter_codes_3 = List[CharField(max_length=128, null=False, blank=False)]
    # name_type_ids = List[models.IntegerField(null=True))]
    # parish_id = models.IntegerField(null=True)
    # place_name_basis_type_ids = List[models.IntegerField(null=True))]
    # place_name_search_string = models.CharField(max_length=256, null=True, blank=True)
    # place_name_search_type = models.CharField(max_length=128, choices=PLACE_NAME_SEARCH_TYPE, default=Contains)
    # place_name_sort_field = models.CharField(max_length=256, null=True, blank=True)
    # place_name_sort_order = models.CharField(max_length=256, null=True, blank=True)
    # province_id = models.IntegerField(null=True)

    def __init__(self):
        self.country_area_filter_ids = None
        self.country_area_names = None
        self.county_id = None
        self.district_id = None
        self.language_filter_codes_3 = None
        self.name_type_ids = None
        self.parish_id = None
        self.place_name_basis_type_ids = None
        self.place_name_search_string = None
        self.place_name_search_type = None
        self.place_name_sort_field = None
        self.place_name_sort_order = 'ascending'
        self.province_id = None

    def get_place_name_search_type(self) -> str:
        if (hasattr(self, 'place_name_search_type')):
            for place_name_search_type in PlaceNameSearchCriteria.PLACE_NAME_SEARCH_TYPE:
                if (place_name_search_type[0] == self.place_name_search_type):
                    return place_name_search_type[1]

        return ''

    def has_search_criteria(self) -> bool:
        return ((self.country_area_filter_ids is not None) or
                (self.country_area_names is not None) or
                (self.county_id is not None) or
                (self.district_id is not None) or
                (self.language_filter_codes_3 is not None) or
                (self.name_type_ids is not None) or
                (self.parish_id is not None) or
                (self.place_name_basis_type_ids is not None) or
                (is_not_empty_string(self.place_name_search_string)) or
                (self.place_name_search_type is not None) or
                (self.province_id is not None))

    def is_place_name_basis_by_id(self) -> bool:
        return (hasattr(self, 'place_name_search_type') and
                (self.place_name_search_type == 'PlaceNameBasisId'))

    def is_place_name_by_id(self) -> bool:
        return (hasattr(self, 'place_name_search_type') and
               ((self.place_name_search_type == 'PlaceNameId') or
                (self.place_name_search_type == 'PlaceNameBasisId')))
