from django.db import models

from ISOF.model.simple_model_query_set import SimpleModelQuerySet
from ISOF.model.web_service.smp_web_service import SmpWebService


class ReferenceFormQuerySet(SimpleModelQuerySet):
    def _fetch_all(self):
        if (not hasattr(self, '_result_cache')) or (self._result_cache is None):
            smp_service = SmpWebService()
            reference_forms_json = smp_service.get_reference_forms()
            reference_forms = ReferenceForm.get_reference_forms_from_json(reference_forms_json)
            self._result_cache = reference_forms

    def filter(self, *args, **kwargs):
        if len(args) > 0:
            search_string = args[0].children[0].children[0][1]
            if search_string:
                reference_forms = []
                self._fetch_all()
                if self._result_cache is not None:
                    for reference_form in self._result_cache:
                        if search_string in reference_form.name:
                            reference_forms.append(reference_form)
                queryset = ReferenceFormQuerySet(ReferenceForm)
                queryset._result_cache = reference_forms
                return queryset
        return self

class ReferenceFormManager(models.Manager):
    def get_queryset(self):
        queryset = ReferenceFormQuerySet(ReferenceForm)
        return queryset


class ReferenceForm(models.Model):
    name = models.CharField(null=False, blank=False, max_length=100)
    is_composition = models.BooleanField(default=False, null=False)
    oldest_year_of_archive_post = models.CharField(null=False, blank=False, max_length=20)
    is_in_book = models.BooleanField(default=False, null=False)

    def __str__(self):
        return self.name

    class Meta:
        base_manager_name = 'objects'
        verbose_name = "referensform"
        verbose_name_plural = "referensformer"
        ordering = ['name']

    # get reference forms from json
    @classmethod
    def get_reference_forms_from_json(cls, reference_forms_json):
        reference_forms = []
        for reference_form_json in reference_forms_json:
            reference_form = ReferenceForm()
            reference_form.name = reference_form_json['name']
            reference_form.is_composition = reference_form_json['is_composition']
            reference_form.oldest_year_of_archive_post = reference_form_json['oldest_year_of_archive_post']
            reference_form.is_in_book = reference_form_json['is_in_book']
            reference_forms.append(reference_form)
        return reference_forms

    # get reference form from json
    @classmethod
    def get_reference_form_from_json(cls, reference_form_json):
        reference_form = ReferenceForm()
        reference_form.name = reference_form_json['name']
        reference_form.is_composition = reference_form_json['is_composition']
        reference_form.oldest_year_of_archive_post = reference_form_json['oldest_year_of_archive_post']
        reference_form.is_in_book = reference_form_json['is_in_book']
        return reference_form
