from django.db.models import QuerySet


class SimpleModelQuerySet(QuerySet):

    def _chain(self):
        self._fetch_all()
        return self

    def _clone(self):
        self._fetch_all()
        return self

    def __iter__(self):
        self._fetch_all()
        self._next = 0
        return self

    def __next__(self):
        self._fetch_all()
        if (self._next < len(self._result_cache)):
            self._next = self._next + 1
            return self._result_cache[self._next - 1]
        else:
            raise StopIteration

    def alias(self, *args, **kwargs):
        """
        Return a query set with added aliases for extra data or aggregations.
        """
        self._fetch_all()
        return self

    def all(self):
        """
        Return a new QuerySet that is a copy of the current one. This allows a
        QuerySet to proxy for a model manager in some cases.
        """
        self._fetch_all()
        return self

    def annotate(self, *args, **kwargs):
        """
        Return a query set in which the returned objects have been annotated
        with extra data or aggregations.
        """
        self._fetch_all()
        return self

    def count(self):
        """
        Perform a SELECT COUNT() and return the number of records as an
        integer.

        If the QuerySet is already fully cached, return the length of the
        cached results set to avoid multiple SELECT COUNT(*) calls.
        """
        self._fetch_all()
        if (self._result_cache is not None):
            return len(self._result_cache)

        return 0

    def create_dictionary(self, objects):
        self._result_cache_dictionary = {}
        if(objects):
            for object in objects:
                self._result_cache_dictionary[object.id] = object

    def difference(self, *other_qs):
        # If the query is an EmptyQuerySet, return it.
        self._fetch_all()
        return self

    def distinct(self, *field_names):
        """
        Return a new QuerySet instance that will select only distinct results.
        """
        self._fetch_all()
        return self

    def exclude(self, *args, **kwargs):
        """
        Return a new QuerySet instance with NOT (args) ANDed to the existing
        set.
        """
        self._fetch_all()
        return self

    def get(self, *args, **kwargs):
        """
        Perform the query and return a single object matching the given
        keyword arguments.
        """
        self._fetch_all()
        obj_id = self.get_id(args, kwargs)

        if self.has_dictionary():
            return self._result_cache_dictionary[obj_id]
        else:
            for obj in self._result_cache:
                if (obj.id == obj_id):
                    return obj

        return self

    def get_id(self, *args, **kwargs):
        """
        Get object id from parameters *args, **kwargs in a call to method QuerySet.get().
        """
        object_id = None

        if(kwargs):
            if 'id' in kwargs:
                object_id = kwargs['id']
            if 'pk' in kwargs:
                object_id = kwargs['pk']

        if args:
            if ((len(args) == 2) and args[1] and isinstance(args[1], dict) and ('id' in args[1])):
                object_id = args[1]['id']
            elif ((len(args) == 2) and args[1] and isinstance(args[1], dict) and ('pk' in args[1])):
                object_id = args[1]['pk']
            else:
                object_id = args[0].children[0][1]

        if isinstance(object_id, str):
            object_id = int(object_id)

        return object_id

    def has_dictionary(self):
        return hasattr(self, "_result_cache_dictionary") and self._result_cache_dictionary is not None

    def none(self):
        self._result_cache = []
        return self

    def intersection(self, *other_qs):
        # If any query is an EmptyQuerySet, return it.
        self._fetch_all()
        return self

    def iterator(self, chunk_size=None):
        """
        An iterator over the results from applying this QuerySet to the
        database. chunk_size must be provided for QuerySets that prefetch
        related objects. Otherwise, a default chunk_size of 2000 is supplied.
        """
        self._fetch_all()
        return self

    def order_by(self, *field_names):
        """Return a new QuerySet instance with the ordering changed."""
        self._fetch_all()
        return self

    def reverse(self):
        """Reverse the ordering of the QuerySet."""
        self._fetch_all()
        return self

    def union(self, *other_qs, all=False):
        # If the query is an EmptyQuerySet, combine all nonempty querysets.
        self._fetch_all()
        return self
