from datetime import datetime
import requests
import threading
from ISOF.configuration import Configuration
from ISOF.model.country_area_filter import CountryAreaFilter
from ISOF.model.county import County
from ISOF.model.data_source.geo_reference_data_source import GeoReferenceDataSource
from ISOF.model.data_source.place_name_data_source import PlaceNameDataSource
from ISOF.model.district import District
from ISOF.model.language_filter import LanguageFilter
from ISOF.model.name_type import NameType
from ISOF.model.place_name import PlaceName
from ISOF.model.place_name_basis import PlaceNameBasis
from ISOF.model.place_name_basis_search_result import PlaceNameBasisSearchResult
from ISOF.model.place_name_basis_type import PlaceNameBasisType
from ISOF.model.place_name_search_result import PlaceNameSearchResult
from ISOF.model.parish import Parish
from ISOF.model.province import Province
from ISOF.model.web import status_code


class PlaceNameWebService(GeoReferenceDataSource, PlaceNameDataSource):
    geo_reference_path = 'geo-reference'
    place_name_basis_path = 'place-name-basis'
    place_names_path = 'place-names'
    web_service_domain = Configuration.PLACE_NAME_SERVICE_ADDRESS
    if (Configuration.computer == Configuration.BEAR_DEVELOPMENT_COMPUTER_NAME):
        web_service_domain = Configuration.PLACE_NAME_SERVICE_TEST_ADDRESS
    if (Configuration.computer == Configuration.JOHAN_DEVELOPMENT_COMPUTER_NAME):
        web_service_domain = Configuration.PLACE_NAME_SERVICE_TEST_ADDRESS
    if (Configuration.computer == Configuration.FRIGG_TEST_NAME):
        web_service_domain = Configuration.PLACE_NAME_SERVICE_TEST_ADDRESS
    if (Configuration.computer == Configuration.FRIGG_NAME):
        web_service_domain = Configuration.PLACE_NAME_SERVICE_ADDRESS
    if (Configuration.computer == Configuration.GARM_TEST_NAME):
        web_service_domain = Configuration.PLACE_NAME_SERVICE_TEST_ADDRESS
    if (Configuration.computer == Configuration.GARM_NAME):
        web_service_domain = Configuration.PLACE_NAME_SERVICE_ADDRESS

    web_service_version = 'v1'
    web_service_url = Configuration.INTERNET_PROTOCOL + web_service_domain + '/'
    geo_reference_basis_url = web_service_url + web_service_version + '/' + geo_reference_path
    place_name_basis_url = web_service_url + web_service_version + '/' + place_name_basis_path
    place_names_url = web_service_url + web_service_version + '/' + place_names_path

    def get_counties(self):
        response = requests.get(PlaceNameWebService.geo_reference_basis_url + '/counties')
        counties_json = response.json()
        counties = []
        if (len(counties_json) > 0):
            for county_json in counties_json:
                county = County()
                county.code = county_json['code']
                county.country_code_2 = county_json['country_code_2']
                county.id = county_json['id']
                county.name = county_json['name']
                county.province_ids = county_json['province_ids']
                counties.append(county)
        return counties

    def get_country_area_filters(self):
        response = requests.get(PlaceNameWebService.place_names_url + '/country-area-filters')
        country_area_filters_json = response.json()
        country_area_filters = []
        if (len(country_area_filters_json) > 0):
            for country_area_filter_json in country_area_filters_json:
                country_area_filter = CountryAreaFilter()
                country_area_filter.code_2 = country_area_filter_json['code_2']
                country_area_filter.code_3 = country_area_filter_json['code_3']
                country_area_filter.id = country_area_filter_json['id']
                country_area_filter.is_rest_of_world = country_area_filter_json['is_rest_of_world']
                country_area_filter.name = country_area_filter_json['name']
                country_area_filter.part_of_country_id = country_area_filter_json['part_of_country_id']
                country_area_filters.append(country_area_filter)
        return country_area_filters

    def get_districts(self):
        response = requests.get(PlaceNameWebService.geo_reference_basis_url + '/districts')
        districts_json = response.json()
        districts = []
        if (len(districts_json) > 0):
            for district_json in districts_json:
                district = District()
                district.country_area_name = district_json['country_area_name']
                district.country_code_2 = district_json['country_code_2']
                district.county_id = district_json['county_id']
                district.id = district_json['id']
                district.name = district_json['name']
                district.province_id = district_json['province_id']
                districts.append(district)
        return districts

    def get_language_filters(self):
        response = requests.get(PlaceNameWebService.place_names_url + '/language-filters')
        language_filters_json = response.json()
        language_filters = []
        if (len(language_filters_json) > 0):
            for language_filter_json in language_filters_json:
                language_filter = LanguageFilter()
                language_filter.code_3 = language_filter_json['code_3']
                language_filter.name = language_filter_json['name']
                language_filters.append(language_filter)
        return language_filters

    def get_name_types(self):
        response = requests.get(PlaceNameWebService.place_names_url + '/name-types')
        name_types_json = response.json()
        name_types = []
        if (len(name_types_json) > 0):
            for name_type_json in name_types_json:
                name_type = NameType()
                name_type.id = name_type_json['id']
                name_type.name = name_type_json['name']
                name_types.append(name_type)
        return name_types

    def get_parishes(self):
        response = requests.get(PlaceNameWebService.geo_reference_basis_url + '/parishes')
        parishes_json = response.json()
        parishes = []
        if (len(parishes_json) > 0):
            for parish_json in parishes_json:
                parish = Parish()
                parish.country_area_name = parish_json['country_area_name']
                parish.country_code_2 = parish_json['country_code_2']
                parish.county_id = parish_json['county_id']
                parish.district_id = parish_json['district_id']
                parish.id = parish_json['id']
                parish.latitude = parish_json['latitude']
                parish.longitude = parish_json['longitude']
                parish.name = parish_json['name']
                parish.province_id = parish_json['province_id']
                parishes.append(parish)
        return parishes

    def get_place_name(self, place_name_id):
        response = requests.get(PlaceNameWebService.place_names_url + '/' + str(place_name_id))
        if (response.status_code == status_code.HTTP_404_NOT_FOUND):
            # No place name has the specified id.
            return None
        place_name_json = response.json()
        return self.__get_place_name(place_name_json)

    def get_place_name_basis(self, place_name_basis_search_criteria=None):
        # Handle parameters.
        parameters = {}
        if (place_name_basis_search_criteria is not None):
            self.__add_parameter(parameters, 'page-number', place_name_basis_search_criteria.page_number)
            self.__add_parameter(parameters, 'page-size', place_name_basis_search_criteria.page_size)
            self.__add_parameter(parameters, 'place-name-id', place_name_basis_search_criteria.place_name_id)
            self.__add_parameter(parameters,
                                 'place-name-basis-type-id',
                                 place_name_basis_search_criteria.place_name_basis_type_ids)
            self.__add_parameter(parameters, 'sort-field', place_name_basis_search_criteria.place_name_basis_sort_field)
            self.__add_parameter(parameters, 'sort-order', place_name_basis_search_criteria.place_name_basis_sort_order)

        response = requests.get(PlaceNameWebService.place_name_basis_url,
                                params=parameters)
        place_name_basis_search_result_json = response.json()
        return self.__get_place_name_basis_search_result(place_name_basis_search_result_json)

    def get_place_name_basis_back_side_image(self, place_name_basis_id):
        response = requests.get(PlaceNameWebService.place_name_basis_url + '/' +
                                str(place_name_basis_id) +
                                '/image/back-side')
        if (response.status_code == status_code.HTTP_200_OK):
            image = response.content
        else:
            image = None
        return image

    def get_place_name_basis_by_file_address(self, file_address):
        response = requests.get(PlaceNameWebService.place_name_basis_url + '/file-address/' + file_address)
        if (response.status_code == status_code.HTTP_404_NOT_FOUND):
            # No place name has the specified file address.
            return None
        place_name_basis_json = response.json()
        return self.__get_place_name_basis(place_name_basis_json)

    def get_place_name_basis_by_id(self, place_name_basis_id):
        response = requests.get(PlaceNameWebService.place_name_basis_url + '/' + str(place_name_basis_id))
        if (response.status_code == status_code.HTTP_404_NOT_FOUND):
            # No place name basis has the specified file address.
            return None
        place_name_basis_json = response.json()
        return self.__get_place_name_basis(place_name_basis_json)

    def get_place_name_basis_front_side_image(self, place_name_basis_id):
        response = requests.get(PlaceNameWebService.place_name_basis_url + '/' + str(place_name_basis_id) + '/image/front-side')
        if (response.status_code == status_code.HTTP_200_OK):
            image = response.content
        else:
            image = None
        return image

    def get_place_name_basis_pdf(self, place_name_basis_id):
        response = requests.get(PlaceNameWebService.place_name_basis_url + '/' + str(place_name_basis_id) + '/pdf')
        if (response.status_code == status_code.HTTP_200_OK):
            pdf = response.content
        else:
            pdf = None
        return pdf

    def get_place_name_basis_types(self):
        response = requests.get(PlaceNameWebService.place_name_basis_url + '/types')
        place_name_basis_types_json = response.json()
        place_name_basis_types = []
        if (len(place_name_basis_types_json) > 0):
            for place_name_basis_type_json in place_name_basis_types_json:
                place_name_basis_type = PlaceNameBasisType()
                place_name_basis_type.id = place_name_basis_type_json['id']
                place_name_basis_type.name = place_name_basis_type_json['name']
                place_name_basis_type.old_name = place_name_basis_type_json['old_name']
                place_name_basis_types.append(place_name_basis_type)
        return place_name_basis_types

    def get_place_names(self,
                        place_name_search_criteria=None,
                        page_number=None,
                        page_size=None,
                        place_name_sort_field=None,
                        place_name_sort_order=None):
        # Handle parameters.
        parameters = {}
        if (place_name_search_criteria is not None):
            if ((place_name_search_criteria.place_name_search_string is not None) and
                (place_name_search_criteria.place_name_search_string.strip())):
                self.__add_parameter(parameters, 'place-name', place_name_search_criteria.place_name_search_string)
                self.__add_parameter(parameters, 'place-name-search-type', place_name_search_criteria.place_name_search_type)

            self.__add_parameter(parameters, 'country-area-filter-id', place_name_search_criteria.country_area_filter_ids)
            self.__add_parameter(parameters, 'country-area-name', place_name_search_criteria.country_area_names)
            self.__add_parameter(parameters, 'county-id', place_name_search_criteria.county_id)
            self.__add_parameter(parameters, 'district-id', place_name_search_criteria.district_id)
            self.__add_parameter(parameters, 'language-filter-code-3', place_name_search_criteria.language_filter_codes_3)
            self.__add_parameter(parameters, 'name-type-id', place_name_search_criteria.name_type_ids)
            self.__add_parameter(parameters, 'parish-id', place_name_search_criteria.parish_id)
            self.__add_parameter(parameters, 'province-id', place_name_search_criteria.province_id)
            self.__add_parameter(parameters, 'place-name-basis-type-id', place_name_search_criteria.place_name_basis_type_ids)
            self.__add_parameter(parameters, 'province-id', place_name_search_criteria.province_id)

        self.__add_parameter(parameters, 'page-number', page_number)
        self.__add_parameter(parameters, 'page-size', page_size)
        self.__add_parameter(parameters, 'sort-field', place_name_sort_field)
        self.__add_parameter(parameters, 'sort-order', place_name_sort_order)

        # Get place names.
        response = requests.get(PlaceNameWebService.place_names_url,
                                params=parameters)
        place_name_search_result_json = response.json()
        return self.__get_place_name_search_result(place_name_search_result_json)

    def get_provinces(self):
        response = requests.get(PlaceNameWebService.geo_reference_basis_url + '/provinces')
        provinces_json = response.json()
        provinces = []
        if (len(provinces_json) > 0):
            for province_json in provinces_json:
                province = Province()
                province.country_code_2 = province_json['country_code_2']
                province.county_ids = province_json['county_ids']
                province.id = province_json['id']
                province.name = province_json['name']
                provinces.append(province)
        return provinces

    def ping(self):
        is_ok = False
        try:
            response = requests.get(PlaceNameWebService.web_service_url + 'ping')
            if (response.status_code == status_code.HTTP_200_OK):
                is_ok_json = response.json()
                is_ok = is_ok_json['is_ok']
        except:
            is_ok = False
        return is_ok

    def update(self):
        update_thread = threading.Thread(target=self.__update)
        update_thread.setDaemon(True)
        update_thread.start()

    def __add_parameter(self, parameters, parameter_name, parameter_value):
        if (parameter_value is not None):
            if (parameter_value is list):
                if (len(parameter_value) == 1):
                    parameters[parameter_name] = parameter_value[0]
                else:
                    identifier_string = '['
                    for identifier in parameter_value:
                        if (len(identifier_string) > 1):
                            identifier_string = identifier_string + ','
                        identifier_string = identifier_string + str(identifier)
                    identifier_string = identifier_string + ']'
                    parameters[parameter_name] = identifier_string
            else:
                parameters[parameter_name] = parameter_value

    def __get_place_name(self, place_name_json):
        place_name = PlaceName()
        for key in place_name_json:
            if (key == 'fields'):
                for field in place_name_json['fields']:
                    if (field['name'] == 'PlaceNameBasisMatch'):
                        if (not hasattr(place_name, 'meta')):
                            place_name.meta = {}
                        place_name.meta['PlaceNameBasisMatch'] = field['value']
            else:
                setattr(place_name, key, place_name_json[key])
        place_name.update_geography()
        return place_name

    def __get_place_name_basis(self, place_name_basis_json):
        place_name_basis = PlaceNameBasis()
        for key in place_name_basis_json:
            setattr(place_name_basis, key, place_name_basis_json[key])

        if (place_name_basis.decision_date):
            place_name_basis.decision_date = datetime.strptime(place_name_basis.decision_date, '%Y-%m-%d')

        if (place_name_basis.has_back_side_image):
            place_name_basis.back_side_image_url = PlaceNameWebService.place_name_basis_url + '/' + \
                                                   str(place_name_basis.id) + \
                                                   '/image/back-side'
        else:
            place_name_basis.back_side_image_url = None

        if (place_name_basis.has_front_side_image):
            place_name_basis.front_side_image_url = PlaceNameWebService.place_name_basis_url + '/' + \
                                                    str(place_name_basis.id) + \
                                                    '/image/front-side'
        else:
            place_name_basis.front_side_image_url = None

        if (place_name_basis.has_pdf):
            place_name_basis.pdf_url = PlaceNameWebService.place_name_basis_url + '/' + \
                                       str(place_name_basis.id) + \
                                       '/pdf'
        else:
            place_name_basis.pdf_url = None

        return place_name_basis

    def __get_place_name_basis_list(self, place_name_basis_list_json):
        place_name_basis = []
        if (len(place_name_basis_list_json) > 0):
            for place_name_basis_json in place_name_basis_list_json:
                place_name_basis_item = self.__get_place_name_basis(place_name_basis_json)
                place_name_basis.append(place_name_basis_item)
        return place_name_basis

    def __get_place_name_basis_search_result(self, place_name_basis_search_result_json):
        search_result = PlaceNameBasisSearchResult()
        search_result.count = int(place_name_basis_search_result_json['count'])
        search_result.page_number = int(place_name_basis_search_result_json['page_number'])
        search_result.page_size = int(place_name_basis_search_result_json['page_size'])
        search_result.place_name_basis = self.__get_place_name_basis_list(place_name_basis_search_result_json['place_name_basis'])
        return search_result

    def __get_place_name_search_result(self, place_name_search_result_json):
        search_result = PlaceNameSearchResult()
        search_result.count = int(place_name_search_result_json['count'])
        search_result.page_number = int(place_name_search_result_json['page_number'])
        search_result.page_size = int(place_name_search_result_json['page_size'])
        search_result.place_names = self.__get_place_names(place_name_search_result_json['place_names'])
        return search_result

    def __get_place_names(self, place_names_json):
        place_names = []
        if (len(place_names_json) > 0):
            for place_name_json in place_names_json:
                place_name = self.__get_place_name(place_name_json)
                place_names.append(place_name)
        return place_names

    def __update(self):
        requests.get(PlaceNameWebService.web_service_url + 'harvest/update')
