import django.contrib.auth.models
import json
import requests
import threading

from django.contrib.auth.models import User
from django.core import serializers
from django.core.exceptions import ValidationError

from ISOF.configuration import Configuration
from ISOF.model.web import status_code

class SmpWebService:
    web_service_domain = Configuration.MEDIEVAL_NAMES_SERVICE_TEST_ADDRESS
    if Configuration.computer == Configuration.BEAR_DEVELOPMENT_COMPUTER_NAME:
        web_service_domain = Configuration.MEDIEVAL_NAMES_SERVICE_TEST_ADDRESS
    if Configuration.computer == Configuration.JOHAN_DEVELOPMENT_COMPUTER_NAME:
        web_service_domain = Configuration.MEDIEVAL_NAMES_SERVICE_TEST_ADDRESS
    if Configuration.computer == Configuration.GARM_TEST_NAME:
        web_service_domain = Configuration.MEDIEVAL_NAMES_SERVICE_TEST_ADDRESS
    if Configuration.computer == Configuration.GARM_NAME:
        web_service_domain = Configuration.MEDIEVAL_NAMES_SERVICE_ADDRESS
    web_service_version = 'v1'
    web_service_url = Configuration.INTERNET_PROTOCOL + web_service_domain + '/' + web_service_version + '/'

    def create_abbreviation(self, abbreviation, token, csrftoken):
        data = {}
        data['abbreviation'] = abbreviation.abbreviation
        data['explanation'] = abbreviation.explanation
        data['comment'] = abbreviation.comment
        data['is_currently_used'] = abbreviation.is_currently_used
        data['category'] = abbreviation.category
        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"abbreviations/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if (response.status_code == status_code.HTTP_201_CREATED):
            abbreviation = response.json()
            return abbreviation
        else:
            return None

    def delete_abbreviation(self, abbreviation, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"abbreviations/{abbreviation.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return (response.status_code == status_code.HTTP_200_OK)

    def get_abbreviations(self):
        response = requests.get(SmpWebService.web_service_url + 'abbreviations/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            abbreviations = response.json()
            return abbreviations
        else:
            return None

    def get_headers(self, token, csrftoken):
        return {'Authorization': 'Token ' + token,
                'Content-Type': r'application/json',
                "X-CSRFToken": csrftoken,
                "accept": "application/json",
                'Cookie': f'csrftoken={csrftoken}'}

    def get_token(self, username, password):
        response = requests.post(
            SmpWebService.web_service_url + 'get-token/',
            json={'username': username, 'password': password})
        if response.status_code == status_code.HTTP_200_OK:
            return response.json()['token']
        else:
            return None

    def get_user(self, token):
        response = requests.get(SmpWebService.web_service_url + 'get-user/', headers={'Authorization': 'Token ' + token,
                                                                                     'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            user_data = response.json()
            user = SmpUser()
            user.username = user_data['username']
            user.id = user_data['id']
            user.first_name = user_data['first_name']
            user.last_name = user_data['last_name']
            user.email = user_data['email']
            user.is_active = user_data['is_active']
            user.is_staff = user_data['is_staff']
            user.is_superuser = user_data['is_superuser']
            user.pk = user_data['id']
            return user
        else:
            return None

    def update_abbreviation(self, abbreviation, token, csrftoken):
        data = {}
        data['abbreviation'] = abbreviation.abbreviation
        data['explanation'] = abbreviation.explanation
        data['comment'] = abbreviation.comment
        data['id'] = abbreviation.id
        data['is_currently_used'] = abbreviation.is_currently_used
        data['category'] = abbreviation.category
        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"abbreviations/{abbreviation.id}/",
                                data=data,
                                headers=self.get_headers(token, csrftoken))
        return (response.status_code == status_code.HTTP_200_OK)

    def get_name_last_elements(self):
        response = requests.get(SmpWebService.web_service_url + 'name-last-elements/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            name_last_elements = response.json()
            return name_last_elements
        else:
            return None

    # Create name_last_element in SmpService.
    def create_name_last_element(self, name_last_element, token, csrftoken):
        data = {}
        data['last_element'] = name_last_element.last_element
        data['sort_order'] = name_last_element.sort_order
        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"name-last-elements/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if (response.status_code == status_code.HTTP_201_CREATED):
            name_last_element = response.json()
            return name_last_element
        else:
            return None

    # Delete name_last_element in SmpService.
    def delete_name_last_element(self, name_last_element, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"name-last-elements/{name_last_element.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return (response.status_code == status_code.HTTP_200_OK)

    # Update name_last_element in SmpService.
    def update_name_last_element(self, name_last_element, token, csrftoken):
        data = {}
        data['last_element'] = name_last_element.last_element
        data['sort_order'] = name_last_element.sort_order
        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"name-last-elements/{name_last_element.id}/",
                                data=data,
                                headers=self.get_headers(token, csrftoken))
        return response.status_code == status_code.HTTP_200_OK

    def create_province(self, province, token, csrftoken):
        data = {}
        data['name'] = province.name
        data['abbreviation'] = province.abbreviation
        data['country'] = province.country
        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"provinces/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if (response.status_code == status_code.HTTP_201_CREATED):
            province = response.json()
            return province
        else:
            return None

    def delete_province(self, province, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"provinces/{province.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return (response.status_code == status_code.HTTP_200_OK)

    def update_province(self, province, token, csrftoken):
        data = {}
        data['name'] = province.name
        data['abbreviation'] = province.abbreviation
        data['country'] = province.country
        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"provinces/{province.id}/",
                                data=data,
                                headers=self.get_headers(token, csrftoken))
        return (response.status_code == status_code.HTTP_200_OK)

    def get_provinces(self):
        response = requests.get(SmpWebService.web_service_url + 'provinces/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            provinces = response.json()
            return provinces
        else:
            return None

    def get_districts(self):
        response = requests.get(SmpWebService.web_service_url + 'districts/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            districts = response.json()
            return districts
        else:
            return None

    # update district
    def update_district(self, district, token, csrftoken):
        data = {}
        data['name'] = district.name
        data['province'] = district.province.id
        data['harad_id_isof'] = district.harad_id_isof
        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"districts/{district.id}/",
                                data=data,
                                headers=self.get_headers(token, csrftoken))
        return response.status_code == status_code.HTTP_200_OK

    # create district
    def create_district(self, district, token, csrftoken):
        data = {}
        data['name'] = district.name
        data['province'] = district.province.id
        data['harad_id_isof'] = district.harad_id_isof
        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"districts/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if response.status_code == status_code.HTTP_201_CREATED:
            district = response.json()
            return district
        else:
            return None

    # delete district
    def delete_district(self, district, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"districts/{district.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return response.status_code == status_code.HTTP_200_OK

    # get parishes
    def get_parishes(self):
        response = requests.get(SmpWebService.web_service_url + 'parishes/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            parishes = response.json()
            return parishes
        else:
            return None

    # update parish
    def update_parish(self, parish, token, csrftoken):
        data = {}
        data['name'] = parish.name
        data['district'] = parish.district.id
        data['parish_id_isof'] = parish.parish_id_isof
        data['province'] = parish.province.id
        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"parishes/{parish.id}/",
                                data=data,
                                headers=self.get_headers(token, csrftoken))
        return response.json()

    # create parish
    def create_parish(self, parish, token, csrftoken):
        data = {}
        data['name'] = parish.name
        data['district'] = parish.district.id
        data['parish_id_isof'] = parish.parish_id_isof
        data['province'] = parish.province.id
        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"parishes/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if response.status_code == status_code.HTTP_201_CREATED:
            parish = response.json()
            return parish
        else:
            return None

    # delete parish
    def delete_parish(self, parish, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"parishes/{parish.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return response.status_code == status_code.HTTP_200_OK

    # methods for lexical name
    def get_lexical_names(self):
        response = requests.get(SmpWebService.web_service_url + 'lexical-names/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            lexical_names = response.json()
            return lexical_names
        else:
            return None

    # get lexical name article text
    def get_lexical_name_article_text(self, lexical_name_id, token, csrftoken):
        response = requests.get(SmpWebService.web_service_url + f"get-lexical-name-article-text/{lexical_name_id}/",
                                headers=self.get_headers(token, csrftoken))
        if response.status_code == status_code.HTTP_200_OK:
            lexical_name_article_text = response.json()
            return lexical_name_article_text
        else:
            return None

    def create_lexical_name(self, lexical_name, token, csrftoken):
        data = {}
        data['sex'] = lexical_name.sex
        data['lexical_name_type'] = lexical_name.lexical_name_type
        data['is_in_book'] = lexical_name.is_in_book
        data['text_in_article'] = lexical_name.text_in_article
        data['last_element'] = lexical_name.last_element
        data['origin'] = lexical_name.origin
        data['detailed_origin'] = lexical_name.detailed_origin
        data['name_type'] = lexical_name.name_type
        data['name'] = lexical_name.name
        data['internal_note'] = lexical_name.internal_note
        data['external_note'] = lexical_name.external_note
        data['pdf_file_link'] = lexical_name.pdf_file_link
        data['is_sex_certain'] = lexical_name.is_sex_certain
        data['uncertain_origin'] = lexical_name.uncertain_origin
        data['name_uncertain'] = lexical_name.name_uncertain
        data['last_element_uncertain'] = lexical_name.last_element_uncertain
        data['lexical_name_type_uncertain'] = lexical_name.lexical_name_type_uncertain
        data['frequency_in_book'] = lexical_name.frequency_in_book
        data['has_only_compound_names'] = lexical_name.has_only_compound_names
        data['name_form_comment'] = lexical_name.name_form_comment
        data['is_public'] = lexical_name.is_public
        data['first_element'] = lexical_name.first_element
        data['first_element_uncertain'] = lexical_name.first_element_uncertain
        data['name_form_comment'] = lexical_name.name_form_comment
        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"lexical-names/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if (response.status_code == status_code.HTTP_201_CREATED):
            lexical_name = response.json()
            return lexical_name
        else:
            return None

    def delete_lexical_name(self, lexical_name, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"lexical-names/{lexical_name.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return (response.status_code == status_code.HTTP_200_OK)

    def update_lexical_name(self, lexical_name, token, csrftoken):
        data = {}
        data['sex'] = lexical_name.sex
        data['lexical_name_type'] = lexical_name.lexical_name_type
        data['is_in_book'] = lexical_name.is_in_book
        data['text_in_article'] = lexical_name.text_in_article
        data['last_element'] = lexical_name.last_element
        data['name'] = lexical_name.name
        data['origin'] = lexical_name.origin
        data['detailed_origin'] = lexical_name.detailed_origin
        data['name_type'] = lexical_name.name_type
        data['internal_note'] = lexical_name.internal_note
        data['external_note'] = lexical_name.external_note
        data['pdf_file_link'] = lexical_name.pdf_file_link
        data['is_sex_certain'] = lexical_name.is_sex_certain
        data['uncertain_origin'] = lexical_name.uncertain_origin
        data['name_uncertain'] = lexical_name.name_uncertain
        data['last_element_uncertain'] = lexical_name.last_element_uncertain
        data['lexical_name_type_uncertain'] = lexical_name.lexical_name_type_uncertain
        data['frequency_in_book'] = lexical_name.frequency_in_book
        data['has_only_compound_names'] = lexical_name.has_only_compound_names
        data['name_form_comment'] = lexical_name.name_form_comment
        data['is_public'] = lexical_name.is_public
        data['first_element'] = lexical_name.first_element
        data['first_element_uncertain'] = lexical_name.first_element_uncertain
        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"lexical-names/{lexical_name.id}/",
                                data=data,
                                headers=self.get_headers(token, csrftoken))
        return response.json()

    # get lexical forms from lexical name id
    def get_lexical_forms_from_lexical_name_id(self, lexical_name_id, token, csrftoken):
        response = requests.get(
            SmpWebService.web_service_url + f"get-lexical-form-by-lexical-name-id/{lexical_name_id}/",
            headers=self.get_headers(token, csrftoken))
        if response.status_code == status_code.HTTP_200_OK:
            lexical_forms = response.json()
            return lexical_forms
        else:
            return None

    # get lexical forms
    def get_lexical_forms(self):
        response = requests.get(SmpWebService.web_service_url + r'lexical-forms/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            lexical_forms = response.json()
            return lexical_forms
        else:
            return None

    # create lexical form
    def create_lexical_form(self, lexical_form, token, csrftoken):
        data = {}
        data['form_type'] = lexical_form.form_type
        data['language'] = lexical_form.language.id
        data['lexical_name'] = lexical_form.lexical_name.id
        data['is_in_book'] = lexical_form.is_in_book
        data['name'] = lexical_form.name

        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"lexical-forms/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if response.status_code == status_code.HTTP_201_CREATED:
            lexical_form = response.json()
            return lexical_form
        else:
            return None

        # update lexical form

    def update_lexical_form(self, lexical_form, token, csrftoken):
        data = {}
        data['form_type'] = lexical_form.form_type
        data['language'] = lexical_form.language.id
        data['lexical_name'] = lexical_form.lexical_name.id
        data['is_in_book'] = lexical_form.is_in_book
        data['name'] = lexical_form.name
        data['id'] = lexical_form.id
        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"lexical-forms/{lexical_form.id}/",
                                data=data,
                                headers=self.get_headers(token, csrftoken))
        return response.json()

    # delete lexical form
    def delete_lexical_form(self, lexical_form, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"lexical-forms/{lexical_form.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return (response.status_code == status_code.HTTP_200_OK)

    # get languages
    def get_languages(self):
        response = requests.get(SmpWebService.web_service_url + r'languages/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            languages = response.json()
            return languages
        else:
            return None

    # update language
    def update_language(self, language, token, csrftoken):
        data = {}
        data['language'] = language.language
        data['abbreviation'] = language.abbreviation
        data['sort_order'] = language.sort_order
        data['is_in_lexical_name'] = language.is_in_lexical_name
        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"languages/{language.id}/",
                                data=data,
                                headers=self.get_headers(token, csrftoken))
        return (response.status_code == status_code.HTTP_200_OK)

    # create language
    def create_language(self, language, token, csrftoken):
        data = {}
        data['language'] = language.language
        data['abbreviation'] = language.abbreviation
        data['sort_order'] = language.sort_order
        data['is_in_lexical_name'] = language.is_in_lexical_name
        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"languages/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if (response.status_code == status_code.HTTP_201_CREATED):
            language = response.json()
            return language
        else:
            return None

    # delete language
    def delete_language(self, language, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"languages/{language.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return (response.status_code == status_code.HTTP_200_OK)

    def get_lexical_names_by_search_string(self, search_string):
        # call web service get lexical names by search string
        response = requests.get(SmpWebService.web_service_url + f"get-lexical-names-by-search-string/{search_string}/",
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            lexical_names = response.json()
            return lexical_names
        else:
            return []

    # generate search form for lexical name
    def generate_search_form_for_lexical_name(self, lexical_name_id, token, csrftoken):
        # call web service get lexical names by search string
        response = requests.get(
            SmpWebService.web_service_url + f"generate-search-forms-for-lexical-name/{lexical_name_id}/",
            headers=self.get_headers(token, csrftoken))
        if response.status_code == status_code.HTTP_200_OK:
            search_form = response.json()
            return search_form
        else:
            return None

    # get sources
    def get_sources(self):
        response = requests.get(SmpWebService.web_service_url + 'sources/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            sources = response.json()
            return sources
        else:
            return None

    # create source
    def create_source(self, source, token, csrftoken):
        data = {}
        data['source_name'] = source.source_name
        data['shelf_mark'] = source.shelf_mark
        data['type'] = source.type
        data['series_number'] = source.series_number
        data['book_number_when_added'] = source.book_number_when_added
        data['is_original'] = source.is_original
        data['abbreviation'] = source.abbreviation
        data['several_volumes'] = source.several_volumes
        data['removed_changed_problem_comment'] = source.removed_changed_problem_comment
        data['reference'] = source.reference
        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"sources/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if response.status_code == status_code.HTTP_201_CREATED:
            source = response.json()
            return source
        else:
            return None

    # update source
    def update_source(self, source, token, csrftoken):
        data = {}
        data['source_name'] = source.source_name
        data['shelf_mark'] = source.shelf_mark
        data['type'] = source.type
        data['series_number'] = source.series_number
        data['book_number_when_added'] = source.book_number_when_added
        data['is_original'] = source.is_original
        data['abbreviation'] = source.abbreviation
        data['several_volumes'] = source.several_volumes
        data['removed_changed_problem_comment'] = source.removed_changed_problem_comment
        data['reference'] = source.reference
        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"sources/{source.id}/",
                                data=data,
                                headers=self.get_headers(token, csrftoken))

        return response.json()

    # delete source
    def delete_source(self, source, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"sources/{source.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return response.status_code == status_code.HTTP_200_OK

    # get cities
    def get_cities(self):
        response = requests.get(SmpWebService.web_service_url + 'cities/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            cities = response.json()
            return cities
        else:
            return None

    def create_city(self, city, token, csrftoken):
        data = {}
        data['abbreviation'] = city.abbreviation
        data['name'] = city.name
        data['province'] = city.province.id
        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"cities/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if response.status_code == status_code.HTTP_201_CREATED:
            city = response.json()
            return city
        else:
            return None

    # update city
    def update_city(self, city, token, csrftoken):
        data = {}
        data['abbreviation'] = city.abbreviation
        data['name'] = city.name
        data['province'] = city.province.id
        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"cities/{city.id}/",
                                data=data,
                                headers=self.get_headers(token, csrftoken))
        return response.json()

    # delete city
    def delete_city(self, city, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"cities/{city.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return response.status_code == status_code.HTTP_200_OK

    def get_monasteries(self):
        response = requests.get(SmpWebService.web_service_url + 'monasteries/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            monasteries = response.json()
            return monasteries
        else:
            return None

    def create_monastery(self, monastery, token, csrftoken):
        data = {}
        data['name'] = monastery.name
        data['abbreviation'] = monastery.abbreviation
        data['province'] = monastery.province.id
        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"monasteries/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if response.status_code == status_code.HTTP_201_CREATED:
            monastery = response.json()
            return monastery
        else:
            return None

    def update_monastery(self, monastery, token, csrftoken):
        data = {}
        data['name'] = monastery.name
        data['abbreviation'] = monastery.abbreviation
        data['province'] = monastery.province.id
        data['comment'] = monastery.comment
        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"monasteries/{monastery.id}/",
                                data=data,
                                headers=self.get_headers(token, csrftoken))
        return response.json()

    def delete_monastery(self, monastery, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"monasteries/{monastery.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return response.status_code == status_code.HTTP_200_OK

    def get_castles(self):
        response = requests.get(SmpWebService.web_service_url + 'castles/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            castles = response.json()
            return castles
        else:
            return None

    def create_castle(self, castle, token, csrftoken):
        data = {}
        data['castle_name'] = castle.castle_name
        data['province'] = castle.province.id
        data['comment'] = castle.comment
        data['abbreviation'] = castle.abbreviation
        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"castles/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if response.status_code == status_code.HTTP_201_CREATED:
            castle = response.json()
            return castle
        else:
            return None

    def update_castle(self, castle, token, csrftoken):
        data = {}
        data['castle_name'] = castle.castle_name
        data['province'] = castle.province.id
        data['comment'] = castle.comment
        data['abbreviation'] = castle.abbreviation
        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"castles/{castle.id}/",
                                data=data,
                                headers=self.get_headers(token, csrftoken))
        return response.json()

    def delete_castle(self, castle, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"castles/{castle.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return response.status_code == status_code.HTTP_200_OK

    # get blocks
    def get_blocks(self):
        response = requests.get(SmpWebService.web_service_url + f"blocks/",
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            blocks = response.json()
            return blocks
        else:
            return None

    # create block
    def create_block(self, block, token, csrftoken):
        data = {}
        data['name_of_block'] = block.name_of_block
        data['city'] = block.city.id
        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"blocks/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if response.status_code == status_code.HTTP_201_CREATED:
            block = response.json()
            return block
        else:
            return None

    # update block
    def update_block(self, block, token, csrftoken):
        data = {}
        data['name_of_block'] = block.name_of_block
        data['city'] = block.city.id
        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"blocks/{block.id}/",
                                data=data,
                                headers=self.get_headers(token, csrftoken))
        return response.json()

    # delete block
    def delete_block(self, block, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"blocks/{block.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return response.status_code == status_code.HTTP_200_OK

    def get_archive_posts(self):
        response = requests.get(SmpWebService.web_service_url + 'archive-posts/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            archive_posts = response.json()
            return archive_posts
        else:
            return None

    def create_archive_post(self, archive_post, token, csrftoken):
        data = {}
        data['internal_note'] = archive_post.internal_note
        data['is_in_book'] = archive_post.is_in_book
        data['is_seal'] = archive_post.is_seal
        data['name_in_source'] = archive_post.name_in_source
        data['register'] = archive_post.register
        data['type'] = archive_post.type
        if archive_post.date_fake is not None:
            data['date_fake'] = archive_post.date_fake.id
        if archive_post.date_original is not None:
            data['date_original'] = archive_post.date_original.id
        if archive_post.date_transcription is not None:
            data['date_transcription'] = archive_post.date_transcription.id
        data['language'] = archive_post.language.id

        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"archive-posts/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if response.status_code == status_code.HTTP_201_CREATED:
            archive_post = response.json()
            return archive_post
        else:
            return None

    def update_archive_post(self, archive_post, token, csrftoken):
        data = {}
        data['internal_note'] = archive_post.internal_note
        data['is_in_book'] = archive_post.is_in_book
        data['is_seal'] = archive_post.is_seal
        data['name_in_source'] = archive_post.name_in_source
        data['register'] = archive_post.register
        data['type'] = archive_post.type
        if archive_post.date_fake is not None:
            data['date_fake'] = archive_post.date_fake.id
        else:
            data['data_fake'] = None
        if archive_post.date_original is not None:
            data['date_original'] = archive_post.date_original.id
        else:
            data['date_original'] = None
        if archive_post.date_transcription is not None:
            data['date_transcription'] = archive_post.date_transcription.id
        else:
            data['date_transcription'] = None
        data['language'] = archive_post.language.id

        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"archive-posts/{archive_post.id}/",
                                data=data,
                                headers=self.get_headers(token, csrftoken))
        return response.json()

    def delete_archive_post(self, archive_post, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"archive-posts/{archive_post.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return response.status_code == status_code.HTTP_200_OK

    def get_cases(self):
        response = requests.get(SmpWebService.web_service_url + 'cases/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            cases = response.json()
            return cases
        else:
            return None

    def create_case(self, case, token, csrftoken):
        data = {}
        data['abbreviation'] = case.abbreviation
        data['case'] = case.case
        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"cases/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if response.status_code == status_code.HTTP_201_CREATED:
            case = response.json()
            return case
        else:
            return None

    def update_case(self, case, token, csrftoken):
        data = {}
        data['abbreviation'] = case.abbreviation
        data['case'] = case.case
        data['id'] = case.id
        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"cases/{case.id}/",
                                data=data,
                                headers=self.get_headers(token, csrftoken))
        return response.json()

    def delete_case(self, case, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"cases/{case.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return response.status_code == status_code.HTTP_200_OK

    def get_name_occurrences(self):
        response = requests.get(SmpWebService.web_service_url + 'name-occurrences/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            name_occurrences = response.json()
            return name_occurrences
        else:
            return None

    # create name occurrence
    def create_name_occurrence(self, name_occurrence, token, csrftoken):
        data = {}
        data['archive_post'] = name_occurrence.archive_post.id
        if name_occurrence.case:
            data['case'] = name_occurrence.case.id
        data['incorrect'] = name_occurrence.incorrect
        data['is_in_book'] = name_occurrence.is_in_book
        data['language'] = name_occurrence.language.id
        data['lexical_name'] = name_occurrence.lexical_name.id
        data['name_archive_post'] = name_occurrence.name_archive_post
        data['type'] = name_occurrence.type
        if name_occurrence.name_last_element:
            data['name_last_element'] = name_occurrence.name_last_element.id
        else:
            data['name_last_element'] = None

        if name_occurrence.person:
            data['person'] = name_occurrence.person.id
        else:
            data['person'] = None

        # if name_lexical
        if name_occurrence.name_lexical:
            data['name_lexical'] = name_occurrence.name_lexical
        else:
            data['name_lexical'] = None

        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"name-occurrences/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if response.status_code == status_code.HTTP_201_CREATED:
            name_occurrence = response.json()
            return name_occurrence
        else:
            return None

    # update name occurrence
    def update_name_occurrence(self, name_occurrence, token, csrftoken):
        data = {}
        data['archive_post'] = name_occurrence.archive_post.id
        if name_occurrence.case:
            data['case'] = name_occurrence.case.id
        data['incorrect'] = name_occurrence.incorrect
        data['is_in_book'] = name_occurrence.is_in_book
        data['language'] = name_occurrence.language.id
        data['lexical_name'] = name_occurrence.lexical_name.id
        data['name_archive_post'] = name_occurrence.name_archive_post
        data['name_lexical'] = name_occurrence.name_lexical
        data['type'] = name_occurrence.type

        if name_occurrence.name_last_element:
            data['name_last_element'] = name_occurrence.name_last_element.id
        else:
            data['name_last_element'] = None

        if name_occurrence.person:
            data['person'] = name_occurrence.person.id
        else:
            data['person'] = None

        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"name-occurrences/{name_occurrence.id}/",
                                data=data,
                                headers=self.get_headers(token, csrftoken))
        return response.json()

    # delete name occurrence
    def delete_name_occurrence(self, name_occurrence, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"name-occurrences/{name_occurrence.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return response.status_code == status_code.HTTP_200_OK

    # get name occurrences by for archive post
    def get_name_occurrences_by_archive_post(self, archive_post_id):
        response = requests.get(
            SmpWebService.web_service_url + f"get-name-occurrences-by-archive-post-id/{archive_post_id}/")
        if response.status_code == status_code.HTTP_200_OK:
            name_occurrences = response.json()
            return name_occurrences
        else:
            return None

    # get citations
    def get_citations(self):
        response = requests.get(SmpWebService.web_service_url + 'source-references/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            citations = response.json()
            return citations
        else:
            return None

    # create citation
    def create_citation(self, citation, token, csrftoken):
        data = {}
        data['source'] = citation.source.id
        data['archive_post'] = citation.archive_post.id
        data['page'] = citation.page
        data['is_in_book'] = citation.is_in_book
        data['internal_note'] = citation.internal_note
        data['external_note'] = citation.external_note
        data['is_in_book'] = citation.is_in_book
        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"source-references/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if response.status_code == status_code.HTTP_201_CREATED:
            citation = response.json()
            return citation
        else:
            return None

    # update citation
    def update_citation(self, citation, token, csrftoken):
        data = {}
        data['source'] = citation.source.id
        data['archive_post'] = citation.archive_post.id
        data['page'] = citation.page
        data['is_in_book'] = citation.is_in_book
        data['internal_note'] = citation.internal_note
        data['external_note'] = citation.external_note
        data['is_in_book'] = citation.is_in_book
        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"source-references/{citation.id}/",
                                data=data,
                                headers=self.get_headers(token, csrftoken))
        return response.json()

    # delete citation
    def delete_citation(self, citation, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"source-references/{citation.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return response.status_code == status_code.HTTP_200_OK

    def get_source_references_by_archive_post(self, archive_post_id):
        response = requests.get(
            SmpWebService.web_service_url + f"get-source-references-by-archive-post-id/{archive_post_id}/")
        if response.status_code == status_code.HTTP_200_OK:
            source_references = response.json()
            return source_references
        else:
            return None

    def get_date_specification(self, date_specification_id):
        response = requests.get(SmpWebService.web_service_url + f'date-specifications/{date_specification_id}/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            date_specification = response.json()
            return date_specification
        else:
            return None

    def get_date_specifications(self):
        response = requests.get(SmpWebService.web_service_url + 'date-specifications/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            date_specifications = response.json()
            return date_specifications
        else:
            return None

    # create date specification
    def create_date_specification(self, date_specification, token, csrftoken):
        data = {}
        data['date_occurrence'] = date_specification.date_occurrence
        data['day_end'] = date_specification.day_end
        data['day_start'] = date_specification.day_start
        data['external_note'] = date_specification.external_note
        data['is_period'] = date_specification.is_period
        data['month_end'] = date_specification.month_end
        data['month_start'] = date_specification.month_start
        data['type_start_date'] = date_specification.type_start_date
        if data['type_start_date'] == "NonValue":
            data['type_start_date'] = None
        data['type_end_date'] = date_specification.type_end_date
        if data['type_end_date'] == "NonValue":
            data['type_end_date'] = None
        data['year_end'] = date_specification.year_end
        data['year_start'] = date_specification.year_start
        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"date-specifications/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if response.status_code == status_code.HTTP_201_CREATED:
            date_specification = response.json()
            return date_specification
        else:
            raise Exception("Misslyckades med att skapa tidsangivelse")

    # update date specification
    def update_date_specification(self, date_specification, token, csrftoken):
        data = {}
        data['date_occurrence'] = date_specification.date_occurrence
        data['day_end'] = date_specification.day_end
        data['day_start'] = date_specification.day_start
        data['external_note'] = date_specification.external_note
        data['is_period'] = date_specification.is_period
        data['month_end'] = date_specification.month_end
        data['month_start'] = date_specification.month_start
        data['type_start_date'] = date_specification.type_start_date
        if data['type_start_date'] == "NonValue":
            data['type_start_date'] = None
        data['type_end_date'] = date_specification.type_end_date
        if data['type_end_date'] == "NonValue":
            data['type_end_date'] = None
        data['year_end'] = date_specification.year_end
        data['year_start'] = date_specification.year_start
        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"date-specifications/{date_specification.id}/",
                                data=data,
                                headers=self.get_headers(token, csrftoken))
        return response.json()

    #create person
    def create_person(self, person, token, csrftoken):
        data = {}
        #convert person.date_of_birth to string
        if person.date_of_birth is not None:
            data['date_of_birth'] = person.date_of_birth.strftime("%Y-%m-%d")
        if person.date_of_death is not None:
            data['date_of_death'] = person.date_of_death.strftime("%Y-%m-%d")
        data['external_note'] = person.external_note
        data['index'] = person.index
        data['internal_note'] = person.internal_note
        data['is_in_book'] = person.is_in_book
        data['name'] = person.name
        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"persons/",
                                    data=data,
                                    headers=self.get_headers(token, csrftoken))

        if response.status_code == status_code.HTTP_201_CREATED:
            person = response.json()
            return person
        else:
            return None

    #update person
    def update_person(self, person, token, csrftoken):
        data = {}
        if person.date_of_birth is not None:
            data['date_of_birth'] = person.date_of_birth.strftime("%Y-%m-%d")
        if person.date_of_death is not None:
            data['date_of_death'] = person.date_of_death.strftime("%Y-%m-%d")
        data['external_note'] = person.external_note
        data['index'] = person.index
        data['internal_note'] = person.internal_note
        data['is_in_book'] = person.is_in_book
        data['name'] = person.name
        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"persons/{person.id}/",
                                    data=data,
                                    headers=self.get_headers(token, csrftoken))
        return response.json()

    #delete person
    def delete_person(self, person, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"persons/{person.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return response.status_code in [status_code.HTTP_200_OK, status_code.HTTP_204_NO_CONTENT]

    # get persons
    def get_persons(self):
        response = requests.get(SmpWebService.web_service_url + 'persons/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            persons = response.json()
            return persons
        else:
            return None

   # save date specification
    def save_date_specification(self, date_specification, token, csrftoken):
        data = {}
        data['date_specification'] = date_specification.date_specification
        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"date-specifications/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if response.status_code == status_code.HTTP_201_CREATED:
            date_specification = response.json()
            return date_specification
        else:
            return None

    # delete date specification
    def delete_date_specification(self, date_specification, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"date-specifications/{date_specification.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return response.status_code == status_code.HTTP_204_NO_CONTENT

    # get source references
    def get_source_references(self):
        response = requests.get(SmpWebService.web_service_url + 'source-references/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            source_references = response.json()
            return source_references
        else:
            return None

    # create source reference
    def create_source_reference(self, source_reference, token, csrftoken):
        data = {}

        data['source'] = source_reference.source.id
        data['archive_post'] = source_reference.archive_post.id
        data['start_page'] = source_reference.start_page
        data['end_page'] = source_reference.end_page
        data['has_roman_page_number'] = source_reference.has_roman_page_number
        data['is_main_source'] = source_reference.is_main_source
        if source_reference.person:
            data['person'] = source_reference.person.id
        data['sdhk_number'] = source_reference.sdhk_number
        data['serial_number'] = source_reference.serial_number
        data['volume_number'] = source_reference.volume_number
        data['author_of_source'] = source_reference.author_of_source
        data['has_verse_number'] = source_reference.has_verse_number
        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"source-references/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if response.status_code == status_code.HTTP_201_CREATED:
            source_reference = response.json()
            return source_reference
        else:
            return None

    # update source reference
    def update_source_reference(self, source_reference, token, csrftoken):
        data = {}
        data['source'] = source_reference.source.id
        data['archive_post'] = source_reference.archive_post.id
        data['start_page'] = source_reference.start_page
        data['end_page'] = source_reference.end_page
        data['has_roman_page_number'] = source_reference.has_roman_page_number
        data['has_verse_number'] = source_reference.has_verse_number
        data['is_main_source'] = source_reference.is_main_source
        if source_reference.person:
            data['person'] = source_reference.person.id
        data['sdhk_number'] = source_reference.sdhk_number
        data['serial_number'] = source_reference.serial_number
        data['volume_number'] = source_reference.volume_number
        data['author_of_source'] = source_reference.author_of_source

        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"source-references/{source_reference.id}/",
                                data=data,
                                headers=self.get_headers(token, csrftoken))
        return response.json()

    # delete source reference
    def delete_source_reference(self, source_reference, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"source-references/{source_reference.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return response.status_code == status_code.HTTP_200_OK

    #get localities
    def get_localities(self):
        response = requests.get(SmpWebService.web_service_url + 'localities/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            localities = response.json()
            return localities
        else:
            return None

    #create locality
    def create_locality(self, locality, token, csrftoken):
        data = {}
        if locality.block is not None:
            data['block'] = locality.block.id
        if locality.castle is not None:
            data['castle'] = locality.castle.id
        if locality.city is not None:
            data['city'] = locality.city.id
        if locality.district is not None:
            data['district'] = locality.district.id
        if locality.external_note is not None:
            data['external_note'] = locality.external_note
        if locality.homestead is not None:
            data['homestead'] = locality.homestead
        if locality.medieval_district is not None:
            data['medieval_district'] = locality.medieval_district
        if locality.medieval_parish is not None:
            data['medieval_parish'] = locality.medieval_parish
        if locality.monastery is not None:
            data['monastery'] = locality.monastery.id
        if locality.parish is not None:
            data['parish'] = locality.parish.id
        if locality.person is not None:
            data['person'] = locality.person.id
        if locality.province is not None:
            data['province'] = locality.province.id

        data['type'] = locality.type

        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"localities/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if response.status_code == status_code.HTTP_201_CREATED:
            locality = response.json()
            return locality
        else:
            return None

    #update locality
    def update_locality(self, locality, token, csrftoken):
        data = {}
        if locality.block is not None:
            data['block'] = locality.block.id
        if locality.castle is not None:
            data['castle'] = locality.castle.id
        if locality.city is not None:
            data['city'] = locality.city.id
        if locality.district is not None:
            data['district'] = locality.district.id
        if locality.external_note is not None:
            data['external_note'] = locality.external_note
        if locality.homestead is not None:
            data['homestead'] = locality.homestead
        if locality.medieval_district is not None:
            data['medieval_district'] = locality.medieval_district
        if locality.medieval_parish is not None:
            data['medieval_parish'] = locality.medieval_parish
        if locality.monastery is not None:
            data['monastery'] = locality.monastery.id
        if locality.parish is not None:
            data['parish'] = locality.parish.id
        if locality.person is not None:
            data['person'] = locality.person.id
        if locality.province is not None:
            data['province'] = locality.province.id

        data['type'] = locality.type.id

        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"localities/{locality.id}/",
                                data=data,
                                headers=self.get_headers(token, csrftoken))
        return response.json()

    #delete locality
    def delete_locality(self, locality, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"localities/{locality.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return response.status_code == status_code.HTTP_200_OK

    #get name occurence article text
    def get_name_occurrence_article_text(self, name_occurrence_id):
        response = requests.get(SmpWebService.web_service_url + f"get-name-occurrence-article-text/{name_occurrence_id}/")
        if response.status_code == status_code.HTTP_200_OK:
            article_text = response.json()
            return article_text
        else:
            return None

    def get_form_catalogs(self):
        response = requests.get(SmpWebService.web_service_url + 'form-catalogs/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            form_catalogs = response.json()
            return form_catalogs
        else:
            return None

    def create_form_catalog(self, form_catalog, token, csrftoken):
        data = {}
        data['internal_note'] = form_catalog.internal_note
        data['external_note'] = form_catalog.external_note
        data['lexical_name'] = form_catalog.lexical_name.id
        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"form-catalogs/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if response.status_code == status_code.HTTP_201_CREATED:
            form_catalog = response.json()
            return form_catalog
        else:
            return None

    def update_form_catalog(self, form_catalog, token, csrftoken):
        data = {}
        data['internal_note'] = form_catalog.internal_note
        data['external_note'] = form_catalog.external_note
        data['lexical_name'] = form_catalog.lexical_name.id
        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"form-catalogs/{form_catalog.id}/",
                                data=data,
                                headers=self.get_headers(token, csrftoken))
        return response.json()

    def delete_form_catalog(self, form_catalog, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"form-catalogs/{form_catalog.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return response.status_code == status_code.HTTP_200_OK

    #get all form catalog names
    def get_form_catalog_names(self):
        response = requests.get(SmpWebService.web_service_url + 'form-catalog-names/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            form_catalog_names = response.json()
            return form_catalog_names
        else:
            return None

    #create form catalog name
    def create_form_catalog_name(self, form_catalog_name, token, csrftoken):

        data = {}
        data['case'] = form_catalog_name.case.id
        data['end_year'] = form_catalog_name.end_year
        data['form_catalog'] = form_catalog_name.form_catalog.id
        data['last_element'] = form_catalog_name.last_element.id
        data['name'] = form_catalog_name.name
        data['start_year'] = form_catalog_name.start_year

        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"form-catalog-names/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if response.status_code == status_code.HTTP_201_CREATED:
            form_catalog_name = response.json()
            return form_catalog_name
        else:
            return None

    #update form catalog name
    def update_form_catalog_name(self, form_catalog_name, token, csrftoken):
        data = {}
        data['case'] = form_catalog_name.case.id
        if form_catalog_name.end_year is not None:
            data['end_year'] = form_catalog_name.end_year
        data['form_catalog'] = form_catalog_name.form_catalog.id
        if form_catalog_name.last_element is not None:
            data['last_element'] = form_catalog_name.last_element.id
        data['name'] = form_catalog_name.name
        data['start_year'] = form_catalog_name.start_year
        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"form-catalog-names/{form_catalog_name.id}/",
                                data=data,
                                headers=self.get_headers(token, csrftoken))
        return response.json()

    def delete_form_catalog_name(self, form_catalog_name, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"form-catalog-names/{form_catalog_name.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return response.status_code == status_code.HTTP_200_OK

    #get person relations
    def get_person_relations(self):
        response = requests.get(SmpWebService.web_service_url + 'person-relations/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            person_relations = response.json()
            return person_relations
        else:
            return None

    #create person relation
    def create_person_relation(self, person_relation, token, csrftoken):
        data = {}
        if person_relation.child is not None:
            data['child'] = person_relation.child.id
        if person_relation.parent is not None:
            data['parent'] = person_relation.parent.id
        if person_relation.person_1 is not None:
            data['person_1'] = person_relation.person_1.id
        if person_relation.person_2 is not None:
            data['person_2'] = person_relation.person_2.id
        data['relation_type'] = person_relation.relation_type
        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"person-relations/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if response.status_code == status_code.HTTP_201_CREATED:
            person_relation = response.json()
            return person_relation
        else:
            return None

    def update_person_relation(self, person_relation, token, csrftoken):
        data = {}
        if person_relation.child is not None:
            data['child'] = person_relation.child.id
        if person_relation.parent is not None:
            data['parent'] = person_relation.parent.id
        if person_relation.person_1 is not None:
            data['person_1'] = person_relation.person_1.id
        if person_relation.person_2 is not None:
            data['person_2'] = person_relation.person_2.id
        data['relation_type'] = person_relation.relation_type
        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"person-relations/{person_relation.id}/",
                                data=data,
                                headers=self.get_headers(token, csrftoken))
        return response.json()

    # Delete person relation.
    def delete_person_relation(self, person_relation, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"person-relations/{person_relation.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return response.status_code == status_code.HTTP_200_OK

    def create_locality_relation(self, locality_relation, token, csrftoken):
        data = {}
        data['article_text'] = locality_relation.article_text
        data['internal_comment'] = locality_relation.internal_comment
        data['locality_relation'] = locality_relation.locality_relation
        data = json.dumps(data)

        response = requests.post(SmpWebService.web_service_url + f"locality-relations/",
                                 data=data,
                                 headers=self.get_headers(token, csrftoken))
        if response.status_code == status_code.HTTP_201_CREATED:
            person_relation = response.json()
            return person_relation
        else:
            return None

    def delete_locality_relation(self, locality_relation, token, csrftoken):
        response = requests.delete(SmpWebService.web_service_url + f"locality-relations/{locality_relation.id}/",
                                   headers=self.get_headers(token, csrftoken))
        return response.status_code == status_code.HTTP_200_OK

    def get_locality_relations(self):
        response = requests.get(SmpWebService.web_service_url + 'locality-relations/',
                                headers={'Content-Type': r'application/json'})
        if response.status_code == status_code.HTTP_200_OK:
            locality_relations = response.json()
            return locality_relations
        else:
            return None

    def update_locality_relation(self, locality_relation, token, csrftoken):
        data = {}
        data['article_text'] = locality_relation.article_text
        data['internal_comment'] = locality_relation.internal_comment
        data['locality_relation'] = locality_relation.locality_relation
        data = json.dumps(data)

        response = requests.put(SmpWebService.web_service_url + f"locality-relations/{locality_relation.id}/",
                                data=data,
                                headers=self.get_headers(token, csrftoken))
        return response.json()


class SmpUser(User):
    def save(self, *args, **kwargs):
        pass

    def has_perm(self, perm, obj=None):
        return True

    def has_module_perms(self, app_label):
        return True
