from django.test import TestCase
from ISOF.model.place_name_basis_search_criteria import PlaceNameBasisSearchCriteria
from ISOF.model.place_name_search_criteria import PlaceNameSearchCriteria
from ISOF.model.web_service.place_name_web_service import PlaceNameWebService
from ISOF.test_settings import NONE_EXISTING_PLACE_NAME_BASIS_ID
from ISOF.test_settings import NONE_EXISTING_PLACE_NAME_ID
from ISOF.test_settings import PLACE_NAME_IDS
from ISOF.test_settings import PLACE_NAME_BASIS_FILE_ADDRESSES
from ISOF.test_settings import PLACE_NAME_BASIS_IDS


class TestPlaceNameWebService(TestCase):
    def setUp(self):
        self.web_service = PlaceNameWebService()

    def tearDownModule(self):
        pass

    def test_constructor(self):
        web_service = PlaceNameWebService()
        self.assertIsNotNone(web_service)

    def test_get_counties(self):
        counties = self.web_service.get_counties()
        self.assertIsNotNone(counties)
        self.assertGreater(len(counties), 18)
        self.assertLess(len(counties), 30)

    def test_get_country_area_filters(self):
        country_area_filters = self.web_service.get_country_area_filters()
        self.assertIsNotNone(country_area_filters)
        self.assertGreater(len(country_area_filters), 4)
        self.assertLess(len(country_area_filters), 10)

    def test_get_districts(self):
        districts = self.web_service.get_districts()
        self.assertIsNotNone(districts)
        self.assertGreater(len(districts), 250)
        self.assertLess(len(districts), 400)

    def test_get_language_filters(self):
        language_filters = self.web_service.get_language_filters()
        self.assertIsNotNone(language_filters)
        self.assertEqual(len(language_filters), 3)

    def test_get_name_types(self):
        name_types = self.web_service.get_name_types()
        self.assertIsNotNone(name_types)
        self.assertEqual(len(name_types), 4)

    def test_get_parishes(self):
        parishes = self.web_service.get_parishes()
        self.assertIsNotNone(parishes)
        self.assertGreater(len(parishes), 2000)
        self.assertLess(len(parishes), 3000)

    def test_get_place_name(self):
        # Get existing place name.
        name = 'Jumkil'
        place_name = self.web_service.get_place_name(PLACE_NAME_IDS[name])
        self.assertIsNotNone(place_name)
        self.assertEqual(place_name.id, PLACE_NAME_IDS[name])
        self.assertEqual(place_name.place_name, name)

        # Get none existing place name.
        place_name = self.web_service.get_place_name(NONE_EXISTING_PLACE_NAME_ID)
        self.assertIsNone(place_name)

    def test_get_place_name_basis(self):
        # Get existing place name basis.
        name = 'Jumkil'
        place_name_basis_search_criteria = PlaceNameBasisSearchCriteria()
        place_name_basis_search_criteria.place_name_id = PLACE_NAME_IDS[name]
        place_name_basis_search_result = self.web_service.get_place_name_basis(place_name_basis_search_criteria)
        self.assertIsNotNone(place_name_basis_search_result)
        self.assertGreater(place_name_basis_search_result.count, 0)
        for place_name_basis_item in place_name_basis_search_result.place_name_basis:
            self.assertEqual(place_name_basis_item.place_name_id, PLACE_NAME_IDS[name])

        # Get second page.
        page_number = 2
        page_size = 10
        place_name_basis_search_criteria.page_number = page_number
        place_name_basis_search_criteria.page_size = page_size
        place_name_basis_search_result = self.web_service.get_place_name_basis(place_name_basis_search_criteria)
        self.assertIsNotNone(place_name_basis_search_result)
        self.assertGreater(place_name_basis_search_result.count, page_size)
        self.assertGreater(place_name_basis_search_result.count, page_number * page_size)
        self.assertEqual(place_name_basis_search_result.page_number, page_number)
        self.assertEqual(place_name_basis_search_result.page_size, page_size)
        for place_name_basis_item in place_name_basis_search_result.place_name_basis:
            self.assertEqual(place_name_basis_item.place_name_id, PLACE_NAME_IDS[name])

        # Get none existing place name basis.
        place_name_basis_search_criteria = PlaceNameBasisSearchCriteria()
        place_name_basis_search_criteria.place_name_id = NONE_EXISTING_PLACE_NAME_ID
        place_name_basis_search_result = self.web_service.get_place_name_basis(place_name_basis_search_criteria)
        self.assertIsNotNone(place_name_basis_search_result)
        self.assertEqual(place_name_basis_search_result.count, 0)

    def test_get_place_name_basis_back_side_image(self):
        # Get existing place name basis image.
        name = 'Ernilsamåla'
        image = self.web_service.get_place_name_basis_back_side_image(PLACE_NAME_BASIS_IDS[name])
        self.assertIsNotNone(image)

        # Get none existing place name basis image.
        image = self.web_service.get_place_name_basis_front_side_image(NONE_EXISTING_PLACE_NAME_BASIS_ID)
        self.assertIsNone(image)

    def test_get_place_name_basis_by_file_address(self):
        # Get place name basis by existing file address.
        name = 'Jumkil'
        place_name_basis = self.web_service.get_place_name_basis_by_file_address(PLACE_NAME_BASIS_FILE_ADDRESSES[name])
        self.assertIsNotNone(place_name_basis)
        self.assertEqual(place_name_basis.place_name_basis, name)

        # Get place name basis by none existing file address.
        file_address = 'Jumkil'
        place_name_basis = self.web_service.get_place_name_basis_by_file_address(file_address)
        self.assertIsNone(place_name_basis)

    def test_get_place_name_basis_by_id(self):
        # Get place name basis by existing id.
        name = 'Jumkil'
        place_name_basis = self.web_service.get_place_name_basis_by_id(PLACE_NAME_BASIS_IDS[name])
        self.assertIsNotNone(place_name_basis)
        self.assertEqual(place_name_basis.place_name_basis, name)

        # Get place name basis by none existing id.
        none_existing_place_name_basis_id = 123456789
        place_name_basis = self.web_service.get_place_name_basis_by_id(none_existing_place_name_basis_id)
        self.assertIsNone(place_name_basis)

    def test_get_place_name_basis_front_side_image(self):
        # Get existing place name basis image.
        name = 'Jumkil'
        image = self.web_service.get_place_name_basis_front_side_image(PLACE_NAME_BASIS_IDS[name])
        self.assertIsNotNone(image)

        # Get none existing place name basis image.
        image = self.web_service.get_place_name_basis_front_side_image(NONE_EXISTING_PLACE_NAME_BASIS_ID)
        self.assertIsNone(image)

    def test_get_place_name_basis_pdf(self):
        # Get existing place name basis image.
        name = 'Jumkil'
        pdf = self.web_service.get_place_name_basis_pdf(PLACE_NAME_BASIS_IDS[name])
        self.assertIsNotNone(pdf)

        # Get none existing place name basis image.
        pdf = self.web_service.get_place_name_basis_pdf(NONE_EXISTING_PLACE_NAME_BASIS_ID)
        self.assertIsNone(pdf)

    def test_get_place_name_basis_types(self):
        place_name_basis_types = self.web_service.get_place_name_basis_types()
        self.assertIsNotNone(place_name_basis_types)
        self.assertEqual(len(place_name_basis_types), 3)

    def test_get_place_names(self):
        name = 'Jumkil'

        # Get all place names.
        place_names_search_result = self.web_service.get_place_names(None)
        self.assertIsNotNone(place_names_search_result)
        self.assertGreater(place_names_search_result.count, 1000000)
        self.assertIsNotNone(place_names_search_result.place_names)
        self.assertGreater(len(place_names_search_result.place_names), 5)

        # Get place names by name and all the different place name search types.
        search_criteria = PlaceNameSearchCriteria()
        search_criteria.place_name_search_string = name
        for search_type in PlaceNameSearchCriteria.PLACE_NAME_SEARCH_TYPE:
            if ((search_type[0] != 'PlaceNameId') and (search_type[0] != 'PlaceNameBasisId')):
                search_criteria.place_name_search_type = search_type[0]
                place_names_search_result = self.web_service.get_place_names(search_criteria)
                self.assertIsNotNone(place_names_search_result)
                self.assertGreater(place_names_search_result.count, 1)
                self.assertIsNotNone(place_names_search_result.place_names)
                self.assertGreater(len(place_names_search_result.place_names), 1)
                if (search_criteria.place_name_search_type != 'Similar'):
                    for place_name in place_names_search_result.place_names:
                        self.assertGreater(place_name.place_name.find(name), -1)

        # Get second page.
        page_number = 2
        page_size = 10
        search_criteria = PlaceNameSearchCriteria()
        search_criteria.place_name_search_string = name
        search_criteria.place_name_search_type = PlaceNameSearchCriteria.PLACE_NAME_SEARCH_TYPE[0][0]
        place_names_search_result = self.web_service.get_place_names(search_criteria, page_number, page_size)
        self.assertIsNotNone(place_names_search_result)
        self.assertGreater(place_names_search_result.count, 1)
        self.assertIsNotNone(place_names_search_result.place_names)
        self.assertGreater(len(place_names_search_result.place_names), 1)
        for place_name in place_names_search_result.place_names:
            self.assertGreater(place_name.place_name.find(name), -1)

        # Get none existing place names.
        search_criteria = PlaceNameSearchCriteria()
        search_criteria.place_name_search_string = 'abcdef'
        search_criteria.place_name_search_type = PlaceNameSearchCriteria.PLACE_NAME_SEARCH_TYPE[0][0]
        place_names_search_result = self.web_service.get_place_names(search_criteria)
        self.assertIsNotNone(place_names_search_result)
        self.assertEqual(place_names_search_result.count, 0)

    def test_get_provinces(self):
        provinces = self.web_service.get_provinces()
        self.assertIsNotNone(provinces)
        self.assertGreater(len(provinces), 20)
        self.assertLess(len(provinces), 30)

    def test_ping(self):
        is_ok = self.web_service.ping()
        self.assertTrue(is_ok)
