// Check if flex boxes should be wraped or not.
// This function extends the functionality of Bootstrap 4 Flexbox.
// The function changes the behaviour of flexboxes when a specified width limit it passed.
const flexBoxWrapLimit = 1000;
function checkFlexBoxWrap() {
	var isFlexBoxWraped;
	var windowWidth;
	
	windowWidth = $(window).width();
	$(".isof-flex-wrap-toggle").each(function(index){
		isFlexBoxWraped = $(this).hasClass('flex-wrap');
		if ((windowWidth < flexBoxWrapLimit) && (!isFlexBoxWraped)) {
			$(this).addClass('flex-wrap');
		}
		if ((windowWidth >= flexBoxWrapLimit) && isFlexBoxWraped) {
			$(this).removeClass('flex-wrap');
		}
	});
}

// Check if footer position should be adjusted.
// There shall never be any space under the footer in the browser.
function checkFooterPosition() {
	var footer;
	var footerBottom;
	var footerPosition;
	var hasViewElement;
	var newFooterTopPosition;
	var positionChange;
	var screenHeight;
	var viewElement;

	// Get footer element.
	footer = $(".isof-footer").first();

	// Get view element whose heights should be adjusted according to footer position.
	// TODO: Change this functionality to work on an array of view elements.
	hasViewElement = $(".adjust-view-with-footer").length;
	if (hasViewElement) {
    	viewElement = $(".adjust-view-with-footer").first();
	}

	// Remove previouse set style attribute by this function.
	footer.removeAttr('style');
	if (hasViewElement) {
    	viewElement.removeAttr('style');
	}

	// Get current view information.
	footerBottom = footer.position().top + footer.outerHeight(true);
	footerPosition = footer.position();
	screenHeight = $(window).outerHeight();
	// alert(footerBottom);
	// alert(screenHeight);
	if (footerBottom < screenHeight) {
		// Move footer down to bottom of the screen.
		positionChange = screenHeight - footerBottom;
		newFooterTopPosition = footerPosition.top + positionChange;
		footer.css({top: newFooterTopPosition, left: 0, position:'absolute', width:'100%'});
	}


	if (hasViewElement && (footerBottom < screenHeight)) {
		// Change view element heights according to footer change.
		newViewElementHeight = viewElement.height() + positionChange + footer.height();
		// alert(positionChange);
		viewElement.css({height: newViewElementHeight});
	}
}

function removeUrlParameter(parameterName, url) {
    var parameterIndex = url.indexOf(parameterName);

    while (parameterIndex !== -1) {
        urlAfterParameter = url.substring(parameterIndex + parameterName.length);
        nextParameterIndex = urlAfterParameter.indexOf("&");
        if (nextParameterIndex !== -1) {
            urlBeforeParameter = url.substring(0, parameterIndex);
            urlAfterParameter = urlAfterParameter.substring(nextParameterIndex + 1);
        }
        else {
            urlBeforeParameter = url.substring(0, parameterIndex - 1 );
            urlAfterParameter = "";
        }
        url = urlBeforeParameter + urlAfterParameter;
        parameterIndex = url.indexOf(parameterName);
    }

    return url;
}

// Update value for parameter in html a tag links.
// If parameter is already in link then it is updated otherwise it is inserted into link webb address.
// Parameter linkElements is the result of a jQuery selection with multiple html a tag links.
function updateUrlParameter(linkElements, parameterName, parameterValue) {
    // alert(parameterName);
    // alert(parameterValue);
    linkElements.each(function(index) {
        var url = $(this).attr("href");

        if (url.includes(parameterName)) {
            url = removeUrlParameter(parameterName, url);
        }

        // TODO: Handle case when parameter list in URL is empty.
        // Character "?" has to be added first in parameter list in that case.
        url = url + "&" + parameterName + "=" + parameterValue;
        $(this).attr("href", url)
        // console.log(index + ": " + $(this).text() + " " + url);
    });
}

$(document).ready(function() {
	checkFlexBoxWrap();
	checkFooterPosition();

	$(window).resize(checkFlexBoxWrap);
	$(window).resize(checkFooterPosition);
});
