Rikstermbankens webbgränssnitt
==============================

Teknik
------

* Python 3
* Django 3.2
* MariaDB 10
* MongoDB 4

Struktur
--------

Huvuddelen av koden ligger i tre Django moduler.
* model (datahantering)
* rikstermbanken_edit (redigeringsgränssnittet, kräver inloggning)
* rikstermbanken_public (publika gränssnittet)

URL till webbapplikationen är uppdela i tre delar.
* */* publika Rikstermbanken
* */admin* användarhantering (Django), kräver inloggning
* */edit* uppdatering av data, kräver inloggning

Förberedelse för installation och/eller driftsättning
-----------------------------------------------------

För att användaren ska ha rättigheter att checka ut bankvalvet från Gogs måste
en ssh-nyckel skapas i användarens hemkatalog på drift- och
testservern och läggas in i Gogs under *Driftsättningsnycklar*. Detta
måste göras för varje användare som ska kunna driftsätta.

Kör `ssh-keygen` och svara blankt på alla frågor (tryck retur), inklusive frågan om lösenordsfras.

```bash
ssh-keygen
```

Gå till `https://vcs.its.uu.se/sprakradet/bankvalvet` i Gogs och gå till Inställningar ->
Driftsättningsnycklar. Där finns en knapp som heter *Lägg till
driftsättningsnyckel*. Under *Titel* skriver du lämpligen ditt eget
användarnamn och drift- eller testserverns namn. I *Innehåll* klistrar
du in innehållet i filen `.ssh/id_rsa.pub` från drift- eller
testservern. Klicka på den gröna knappen.

IT-support ser till att MongoDB är installerat på servern.

Första installationen
---------------------

På test- och driftservrar installeras webbgränssnittet i
/var/www/django/RikstermbankenDjango/RikstermbankenDjango:

```bash
cd /var/www/django/RikstermbankenDjango
git clone git@vcs.its.uu.se:isof-devs/RikstermbankenDjango.git 
```

Detta kommer att hämta den senast incheckade versionen.

Kopiera lämplig version av konfigurationsfiler lokalt på servern med hjälp av uppdateringsskriptet.
Exemplet är anpassat för testservern. Det finns inga konfigurationsfiler för produktion ännu så de behöver skapas.
```bash
/var/www/django/RikstermbankenDjango/RikstermbankenDjango/update-rikstermbanken-test.sh
```


Python virtual environment ska användas och läggs i katalogen /var/www/django/RikstermbankenDjango/venv
Installera de paket som anges i filen
/var/www/django/RikstermbankenDjango/RikstermbankenDjango/requirements.txt
Skapa en länk (som används av conf-filen) till Python virtual environment med kommandot:
```bash
cd /var/www/django/RikstermbankenDjango
ln -snf venv current_venv
```

IT-support får se till att conf-filen för WSGI
/var/www/django/RikstermbankenDjango/RikstermbankenDjango/RikstermbankenDjango.conf läggs på rätt ställe.


Skapa MongoDB-konto för applikationen (logga in med det adminkonto du
har fått från IT-support).

Kommandot för att logga in som användaren `admin` är:
```bash
mongo -u admin
```

Vid MongoDB:s kommandoprompt kör du följande kommandon (i exemplet heter databasen
`rikstermbanken`, och användarnamnet du skapar är `rb`):

```javascript
    use rikstermbanken
    db.createUser(
       {
         user: "rb",
         pwd: "<hitta på ett lösenord>",
		 roles: [ { role: "readWrite", db: "rikstermbanken" }]
	   }
	)
    db.grantRolesToUser("rb", [ { role: "readWrite", db: "rikstermbanken_log" } ])
    db.grantRolesToUser("rb", [ { role: "readWrite", db: "rikstermbanken_staging" } ])
```

Detta ger användaren `rb` rättigheter i
`rikstermbanken`, `rikstermbanken_staging` och `rikstermbanken_log`.

Lösenord skapas lämpligen med kommandot `openssl rand -base64 21`.


Filen `/var/www/django/RikstermbankenDjango/RikstermbankenDjango/model/mongodbcfg.py` ska ha ett
innehåll enligt följande mall, med samma användarnamn, lösenord och databas som ovan:

```python
    MONGODB_USERNAME = "rb"
    MONGODB_PASSWORD = "<lösenord>"
    MONGODB_DB = "<databasen>"
```

Databasen måste också få ett innehåll. Detta sker genom att läsa in
innehållet i [Bankvalvet](https://vcs.its.uu.se/magah633/rikstermbanken-termlistor)
med redigeringsgränssnittet i webbapplikationen Rikstermbanken (https://Rikstermbanken-Django-test.isof.se/edit/import).

Hämta Bankvalvet:

```bash
cd /var/www/django/RikstermbankenDjango
git clone git@vcs.its.uu.se:sprakradet/bankvalvet.git
```

Skapa filen `/var/www/django/RikstermbankenDjango/RikstermbankenDjango/model/admincfg.py` med inloggningsuppgifter till adminsidan:

```python
ADMIN_USERNAME="admin"
ADMIN_PASSWORD="<lägg in ett lösenord här>"
UPDATE_HOOK_FILE="/var/www/django/RikstermbankenDjango/daemoncontrol/update"
```

`UPDATE_HOOK_FILE` talar om var filen ligger som triggar en
uppdatering. Filen uppdateras genom att Gogs gör en POST till
`https://rikstermbanken-test.isof.se/update-hook`, eller motsvarande i
produktion. För att få Gogs att göra detta läggs en "Webbhook" in
under inställningar i `sprakradet/bankvalvet`. Webbhooken använder
inte innehållet i POST:en, utan kommer att ändra på `UPDATE_HOOK_FILE`
oavsett vad den får från Gogs. En webbhook var för test respektive
produktion måste läggas in separat.

`UPDATE_HOOK_FILE` bevakas av Unixdemonen `update-daemon.py`, som
ligger i repot RikstermbankenDjango, men måste kopieras till det
ställe Uppsala IT kör den ifrån, normalt sett
`/var/www/django/RikstermbankenDjango/`. När `UPDATE_HOOK_FILE` ändras kör
`update-daemon.py` först `git pull` och sedan en import. Utmatningen
loggas i `/var/www/django/RikstermbankenDjango/update-bankvalvet.log`. Om inget
händer i Gogs körs uppdateringar var 15:e minut. För att felsöka detta
kan man även tvinga Gogs att skicka en POST, detta görs genom knappen
"Testa leverans" inne på webbhooksidan. Då får man också reda på om
det gick bra eller inte.

`UPDATE_HOOK_FILE` måste göras skrivbar för alla:

```bash
chmod 666 /var/www/rikstermbanken/daemoncontrol/update
```

Tillsvidare är detta bara gjort för rikstermbanken-test, eftersom
Uppsala IT måste lägga in att demonen körs, vilket inte är gjort i
produktion än. Istället körs uppdateringar varje kvart från ett
cron-script:

```bash
05,20,35,50 * * * * /var/www/rikstermbanken/src/update-bankvalvet.sh >> /var/www/rikstermbanken/update-bankvalvet.log 2>&1
```

Demonen kräver pythonpaketet `watchdog`, vilket måste installeras:

```bash
sudo /bin/pip3.6 install watchdog
```


Driftsättning
-------------

Detta görs varje gång en ny version ska driftsättas, först på
testservern, och efter testningen är gjord, på samma sätt på driftservern:

```bash
    /var/www/django/RikstermbankenDjango/RikstermbankenDjango/update-rikstermbanken-test.sh
```
Starta om Apache.

Databassökningar och prestanda
------------------------------

För att köra kommandon vid mongo-kommandoprompten på test- eller
driftservern är det lämpligast att så långt som möjligt vara inloggad
som `rb`. Dock saknar den användaren skrivrättigheter, så för att
skapa index behöver man vara inloggad som `admin`. Användaren `rb` får
(om användaren är skapad enligt ovan) bara logga in i databasen
`rikstermbanken`, så kommandot för att starta programmet blir:

```bash
    mongo -u rb rikstermbanken
```

Det görs för närvarande tre olika sökningar i databasen.
Den första söker efter den söksträng (`searchString`) användaren har matat in. Den ser ut så
här i Rikstermbankens pythonkod:

```python
    collation = Collation("sv@collation=search", strength=2)
    results = db.termpost.find({
        "kalla.status": 2,
        "searchterms": searchString
        }, collation=collation)
```
		
Anledningen till att `collation` används är för att göra sökningen
skiftlägesokänslig. Den första parametern till `Collation` är språket,
och att strength är `2` betyder att den inte ska göra skillnad på
skiftläge: [se ICU-dokumentationen](http://userguide.icu-project.org/collation/concepts#TOC-Comparison-Levels).

Samma sökning ser ut så här om man vill göra den vid mongo-kommandoprompten:

```javascript
    db.termpost.find( {
        "kalla.status": 2,
        "searchterms": "<searchString>"
        } ).collation( { locale: "sv@collation=search", strength: 2 } )
```

Genom att lägga till `.pretty()` allra sist så får man utmatningen snyggt formaterad.

För att kontrollera hur lång tid sökningen tar och om indexen används
korrekt kan man lägga på .explain("executionStats"):

```javascript
    db.termpost.find( {
        "kalla.status": 2,
        "searchterms": "<searchString>"
        } ).collation( { locale: "sv@collation=search", strength: 2 } ).explain("executionStats")
```

De intressanta delarna är under avdelningen `executionStats`. Här ett
exempel när vi inte har ett lämpligt index:

```json
	"executionStats" : {
		"executionSuccess" : true,
		"nReturned" : 8,
		"executionTimeMillis" : 263,
		"totalKeysExamined" : 0,
		"totalDocsExamined" : 154122,
		"executionStages" : {
			"stage" : "COLLSCAN",
			"filter" : {
				"$and" : [
					{
						"kalla.status" : {
							"$eq" : 2
						}
					},
					{
						"searchterms" : {
							"$eq" : "e-post"
						}
					}
				]
			},
			"nReturned" : 8,
			"executionTimeMillisEstimate" : 2,
			"works" : 154124,
			"advanced" : 8,
			"needTime" : 154115,
			"needYield" : 0,
			"saveState" : 1204,
			"restoreState" : 1204,
			"isEOF" : 1,
			"direction" : "forward",
			"docsExamined" : 154122
		}
	}
```

Här kan vi se att sökningen tog 263 millisekunder
(`executionTimeMillis`) och att antalet dokument som behövde gås
igenom var 154122 (`totalDocsExamined`). Anledningen till det kan vi
se under `executionStages`, där `stage` är `COLLSCAN` (collection
scan), motsvarande en table scan i SQL.

Vi skapar ett index för den här sökningen:

```javascript
    db.termpost.createIndex({"searchterms":1}, { collation: { locale: "sv@collation=search", strength: 2 } })
```

Här är `searchterms` det fält som ska indexeras, och `collation` är
samma som vi utförde sökningen med ovan. Att det står `1` efter
searchterms betyder att indexet är stigande.

Om det redan finns ett index på searchterms så kan man ta bort det
med:

```javascript
    db.termpost.dropIndex({"searchterms":1})
```

Om vi nu gör om samma sökning igen får vi följande resultat:

```json
	"executionStats" : {
		"executionSuccess" : true,
		"nReturned" : 8,
		"executionTimeMillis" : 9,
		"totalKeysExamined" : 9,
		"totalDocsExamined" : 9,
		"executionStages" : {
			"stage" : "FETCH",
			"filter" : {
				"kalla.status" : {
					"$eq" : 2
				}
			},
			"nReturned" : 8,
			"executionTimeMillisEstimate" : 0,
			"works" : 10,
			"advanced" : 8,
			"needTime" : 1,
			"needYield" : 0,
			"saveState" : 0,
			"restoreState" : 0,
			"isEOF" : 1,
			"docsExamined" : 9,
			"alreadyHasObj" : 0,
			"inputStage" : {
				"stage" : "IXSCAN",
				"nReturned" : 9,
				"executionTimeMillisEstimate" : 0,
				"works" : 10,
				"advanced" : 9,
				"needTime" : 0,
				"needYield" : 0,
				"saveState" : 0,
				"restoreState" : 0,
				"isEOF" : 1,
				"keyPattern" : {
					"searchterms" : 1
				},
				"indexName" : "searchterms_1",
				"collation" : {
					"locale" : "sv@collation=search",
					"caseLevel" : false,
					"caseFirst" : "off",
					"strength" : 2,
					"numericOrdering" : false,
					"alternate" : "non-ignorable",
					"maxVariable" : "punct",
					"normalization" : true,
					"backwards" : false,
					"version" : "57.1"
				},
				"isMultiKey" : true,
				"multiKeyPaths" : {
					"searchterms" : [
						"searchterms"
					]
				},
				"isUnique" : false,
				"isSparse" : false,
				"isPartial" : false,
				"indexVersion" : 2,
				"direction" : "forward",
				"indexBounds" : {
					"searchterms" : [
						"[\"1\u0005\u000eGEMO\u0001\n\", \"1\u0005\u000eGEMO\u0001\n\"]"
					]
				},
				"keysExamined" : 9,
				"seeks" : 1,
				"dupsTested" : 9,
				"dupsDropped" : 0
			}
		}
	}
```

Nu tog det istället 9 millisekunder, och totalt 9 dokument gicks igenom.

En annan sökning som görs är för att hitta en specifik termpost. I
Rikstermbankens pythonkod ser den ut så här:

```javascript
    termpost = db.termpost.find_one({
        "id": id,
        })
```

Motsvarande mongo-kommandovariant blir:

```javascript
    db.termpost.findOne({id: <termpostid>})
```

För att den här sökningen ska gå snabbt behövs detta index:

```javascript
    db.termpost.createIndex({"id":1})
```

Till sist har vi en sökning för att hitta vilket termpostid en länkad
term (seealso, seeunder) har givet termen, källan och ett eventuellt homonymnummer:

```python
    if homonym:
        search = {"term":term, "homonym":homonym}
    else:
        search = {"term":term, "homonym":{"$exists":False}}
    return db.termpost.find_one({"terms":{"$elemMatch":search}, "kalla.id":kalla_id})
```

Sökningen görs bland termpostens alla termer. Om homonyminformationen
inte hade varit viktig hade sökningen kunnat se ut så här:

```javascript
    db.termpost.findOne( { "terms.term": "<term>", "kalla.id": "<källaid>" } )
```

Men eftersom det kan finnas flera termer, och det inte räcker med att
termsträngen stämmer med en term och homonymnumret med en annan så
måste de matchas samtidigt:

```javascript
    db.termpost.findOne( { "terms": {
        "$elemMatch": {
	        "term": "<term>",
            "homonym": "<homonym>"
        }
    }, "kalla.id": "<källaid>" } )
```

eller om homonymnumret inte ska finnas:

```javascript
    db.termpost.findOne( { "terms": {
        "$elemMatch": {
	        "term": "<term>",
            "homonym": { "$exists" : False }
        }
    }, "kalla.id": "<källaid>" } )
```

En sökning för att hitta högsta källid, nästa källid skall alltså vara svaret+1

```javascript
    db.termpost.find().sort({"kalla.id":-1}).limit(1).toArray().map(function(u){return u.kalla.id})
```

robots.txt och sitemap.txt
------
För att underlätta för sökmotorer att indexera sidorna och referera trafik korrekt har en manuell rutin för generering av en robots.txt och en sitemap.txt.
* Skapa filen sitemap.txt och sätt rättigheter så att webservern kan skriva till den.
```bash
    touch sitemap.txt
    chmod 777 sitemap.txt
```