import itertools
from model.iftexts import iftexts
from model.view_type import ViewType


def add_render_information(languages):
    for index, language in enumerate(languages):
        for text_row in language['texts']:
            for paragraph in text_row['paragraphs']:
                paragraph['formated_text'] = get_formated_text(paragraph)
            text_row['text_type'] = iftexts['texttype'][text_row['type']]

        for term_group in language['termgroups']:
            if (term_group['key'].termtype == "formula"):
                if (len(term_group['group']) > 1):
                    term_group['term_group_formula'] = iftexts['formula']['pl'][term_group['key'].level]
                else:
                    term_group['term_group_formula'] = iftexts['formula']['sg'][term_group['key'].level]
            elif (term_group['key'].termtype == "phrase"):
                if (len(term_group['group']) > 1):
                    term_group['term_group_phrase'] = iftexts['lang']['pl'][language['lang']] + ' ' + \
                                                      iftexts['phrase']['pl'][term_group['key'].level]
                else:
                    term_group['term_group_phrase'] = iftexts['lang']['sg'][language['lang']] + ' ' + \
                                                      iftexts['phrase']['sg'][term_group['key'].level]
            else:
                if (len(term_group['group']) > 1):
                    term_group['term_group_term'] = iftexts['lang']['pl'][language['lang']] + ' ' + \
                                                    iftexts['term']['pl'][term_group['key'].level]
                else:
                    term_group['term_group_term'] = iftexts['lang']['sg'][language['lang']] + ' ' + \
                                                    iftexts['term']['sg'][term_group['key'].level]

            for term in term_group['group']:
                term['term_post_name'] = get_term_post_name(term)
                if ('full form' in term):
                    term['full_form'] = True
                if ('gender' in term):
                    term['formated_gender'] = term['gender'].replace(' eller ', r'/')
                if (('pos_not_obvious' in term) and term['pos_not_obvious']):
                    if (('pos' in term) and (term['pos'] is not None) and (term['pos'] != '')):
                        pos = term['pos']
                    else:
                        pos = 'okänd'
                    term['position'] = pos

        # if ((index % 2) == 0):
            # language['css_class'] = 'languageBandOdd'
        # else:
            # language['css_class'] = 'languageBandEven'

        language['css_class'] = 'languageBandOdd'


def get_formated_annotated_text(annotated_text, plain_text=False):
    formated_annotated_text = ''
    for item in annotated_text:
        if ('text' in item):
            formated_annotated_text = formated_annotated_text + item['text']
        elif ('tag' in item):
            if (plain_text):
                formated_annotated_text = formated_annotated_text + get_formated_annotated_text(item['content'],
                                                                                                plain_text=plain_text)
            else:
                formated_annotated_text = formated_annotated_text + \
                                          '<' + item['tag'] + '>' + \
                                          get_formated_annotated_text(item['content']) + \
                                          '</' + item['tag'] + '>'
        elif ('term' in item):
            formated_annotated_text = formated_annotated_text + item['term']['term'] + ' (' + item['term']['homonym'] + ')'
    return formated_annotated_text


def get_formated_text(paragraph, plain_text=False):
    if ('text' in paragraph):
        return paragraph['text']
    else:
        return get_formated_annotated_text(paragraph['annotated_text'], plain_text=plain_text)


def get_sort_order(item_list):
    return {value: index for index, value in enumerate(item_list)}


term_type_sort_order = get_sort_order([
   ViewType("rec", "term"), ViewType("std", "term"), ViewType("acc", "term"),
   ViewType("dep", "term"), ViewType("inf", "term"), ViewType("rec", "phrase"),
   ViewType("std", "phrase"), ViewType("acc", "phrase"), ViewType("dep", "phrase"),
   ViewType("rec", "formula"), ViewType("std", "formula"), ViewType("acc", "formula"),
   ViewType("dep", "formula"),
])

text_type_sort_order = get_sort_order(["domain", "context", "definition", "explanation", "comment", "language_usage_explanation"])


def get_term_post_name(term):
    term_post_name = ''
    if ('term' in term):
        term_post_name = term_post_name + term['term']
        if ('homonym' in term):
            term_post_name = term_post_name + ' (' + str(term['homonym']) + ')'
    else:
        if ('annotated_term' in term):
            for annotated_term in term['annotated_term']:
                term_post_name = term_post_name + get_term_post_name_annotated([annotated_term])
                if ('text' in annotated_term):
                    if (term_post_name != ''):
                        term_post_name = term_post_name + ' '
                    term_post_name = term_post_name + annotated_term['text']

    term['term_post_name'] = term_post_name
    return term_post_name


def get_term_post_name_annotated(annotated_terms):
    term_post_name = ''
    for annotated_term in annotated_terms:
        if ('text' in annotated_term):
            term_post_name = term_post_name + annotated_term['text']
        elif ('tag' in annotated_term):
            term_post_name = '<' + annotated_term['tag'] + '>' + \
                             get_term_post_name_annotated(annotated_term['content']) + \
                             '</' + annotated_term['tag'] + '>'
    return term_post_name


def get_terms_by_language(term_post, language_code):
    terms = [term for term in term_post.get("terms", []) if term["lang"] == language_code]
    for term in terms:
        term["viewtype"] = get_term_type(term)
    grouped_terms = group_by(terms, lambda x: x["viewtype"])
    grouped_terms = sorted(grouped_terms, key=lambda x: term_type_sort_order.get(x["key"], 100))
    texts = [text for text in term_post.get("texts", []) if text["lang"] == language_code]
    texts = sorted(texts, key=lambda x: text_type_sort_order.get(x["type"], 100))
    result = {"termgroups": grouped_terms, "texts": texts, "lang": language_code}
    return result


def get_term_type(term):
    if term.get("phrase", False):
        term_or_phrase = "phrase"
    else:
        term_or_phrase = "term"
    if term.get("split", False):
        if term["status"] == "TE":
            return ViewType("rec", term_or_phrase)
        elif term["status"] == "BT":
            return ViewType("rec", "formula")
        elif term["status"] == "SYTE":
            return ViewType("acc", term_or_phrase)
    if term["status"] in ["TE", "SYTE"]:
        return ViewType("std", term_or_phrase)
    elif term["status"] == "BT":
        return ViewType("std", "formula")
    elif term["status"] == "AVTE":
        return ViewType("dep", term_or_phrase)
    elif term["status"] == "INTE":
        return ViewType("inf", "term")
    else:
        return ViewType("std", term_or_phrase)  # XXX


def group_by(terms, key) -> list:
    """ Group terms based on values for the field specified in parameter key.

    Arguments:
        terms (dic): Terms.
        key ()

    Returns:
        List: List with groups of terms.
    """
    groups = []
    for key, group in itertools.groupby(sorted(terms, key=key), key):
        groups.append({"key": key, "group": list(group)})
    return groups
