from bson.regex import Regex
import re
from typing import Any, NamedTuple


def glob_to_regex(s, pythonstyle=False):
    escaped = ""
    for c in s:
        if c == "*":
            escaped += ".*"
        elif c == "?":
            escaped += "."
        else:
            if pythonstyle:
                escaped += "\\u%04x" % ord(c)
            else:
                escaped += "\\x{%04x}" % ord(c)
    if pythonstyle:
        return re.compile("^" + escaped + "$", re.IGNORECASE)
    else:
        return Regex("^" + escaped + "$", flags="i")


class RegexSearchStringExpression(NamedTuple):
    pythonregex: Any
    bsonregex: Regex

    @classmethod
    def create(cls, s):
        bsonregex = glob_to_regex(s)
        pythonregex = glob_to_regex(s, pythonstyle=True)
        return cls(pythonregex=pythonregex, bsonregex=bsonregex)

    def mongodbsearchversion(self):
        return self.bsonregex

    def isString(self):
        return False

    def matches(self, s):
        return self.pythonregex.match(s) is not None

    def matchesAny(self, l):
        return any([self.matches(e) for e in l])

