from django.conf import settings
import glob
from ISOF.common.string_functions import is_integer_string
import os.path


class SourceFileManager():
    """ Handles files with information about term posts. """

    @staticmethod
    def get_file_name(file_path: str) -> str:
        file_name = os.path.basename(file_path)
        return file_name

    @staticmethod
    def get_file_names():
        file_names = []

        for file_path in SourceFileManager.get_text_file_paths():
            file_names.append(SourceFileManager.get_file_name(file_path))

        for file_path in SourceFileManager.get_json_file_paths():
            file_names.append(SourceFileManager.get_file_name(file_path))

        file_names.sort()
        return file_names

    @staticmethod
    def get_json_file_paths():
        return glob.glob(settings.BANKVALVET_DIRECTORY + r'oklara/*.json', recursive=True)

    @staticmethod
    def get_next_free_source_id() -> int:
        max_source_id = -1

        for file_path in SourceFileManager.get_text_file_paths():
            with open(file_path, encoding="utf-8") as file:
                for row in file:
                    row_parts = row.split(" ")
                    if ((len(row_parts) == 2) and (row_parts[0] == 'Källid') and is_integer_string(row_parts[1])):
                        source_id = int(row_parts[1])
                        if (max_source_id < source_id):
                            max_source_id = source_id
                        break

        for file_path in SourceFileManager.get_json_file_paths():
            with open(file_path, encoding="utf-8") as file:
                for row in file:
                    row_parts = row.strip().split(" ")
                    if ((len(row_parts) == 2) and (row_parts[0] == '"id":') and is_integer_string(row_parts[1][0:-1])):
                        source_id = int(row_parts[1][0:-1])
                        if (max_source_id < source_id):
                            max_source_id = source_id
                        break

        return max_source_id + 1

    @staticmethod
    def get_short_file_path(file_path):
        bankvalvet_index = file_path.find('bankvalvet')
        if (bankvalvet_index != -1):
            return file_path[bankvalvet_index:]
        else:
            return file_path

    @staticmethod
    def get_text_file_paths():
        return glob.glob(settings.BANKVALVET_DIRECTORY + r'**/*.txt', recursive=True)

    @staticmethod
    def get_updated_files(file_paths, term_post_sources):
        updated_files = []

        for file_path in file_paths:
            file_name = SourceFileManager.get_file_name(file_path)
            if (file_name in term_post_sources):
                term_post_source = term_post_sources[file_name]
                if ('last_file_change_seconds' in term_post_source):
                    file_information = os.stat(file_path)
                    last_file_change_seconds = file_information.st_mtime
                    source_staging_change_seconds = term_post_source['last_file_change_seconds']
                    if (last_file_change_seconds <= source_staging_change_seconds):
                        # Source file has not changed.
                        continue

            # Source file has changed or not enough information to test if that is the case.
            updated_files.append(file_path)

        return updated_files

