import datetime
import pytz
from django.contrib.auth.decorators import login_required
from django.shortcuts import redirect, render
from ISOF.common.list_functions import is_not_empty_list
from model.database import Database
from model.functions import add_render_information, get_formated_text, get_term_post_name, get_terms_by_language
from model.iftexts import iftexts
from model.import_util import sort_lang_codes
from model.source_file_manager import SourceFileManager
from django.core.files import File
import subprocess
import threading
from urllib.parse import urlencode
from model.term_manager import TermManager
from rikstermbanken_edit.models import Help_texts


@login_required
def compare_databases(request):
    context = {}
    compare_result = TermManager.get_comparison()
    diffs, other_errors = compare_result
    if (diffs is not None):
        new_diffs = []
        for source_id, source_filename, source_diff, errors in diffs:
            if ('added_terms' in source_diff):
                slugs = source_diff['added_terms']
                added_terms = []
                for slug in slugs:
                    term_post = TermManager.get_term_post_by_slug(slug, source_id, 'staging')
                    term_post = render_termpost(term_post)
                    add_render_information(term_post['languages'])
                    source = TermManager.get_term_post_source(term_post['kalla']['id'])
                    term_post['is_source_in_public'] = source is not None
                    added_terms.append(term_post)
                source_diff['added_terms'] = added_terms

            if ('changed_terms' in source_diff):
                for changed_terms in source_diff['changed_terms']:
                    for changed_term in changed_terms:
                        slug = changed_term['title']
                        term_post = TermManager.get_term_post_by_slug(slug, source_id, 'staging')
                        term_post = render_termpost(term_post)
                        add_render_information(term_post['languages'])
                        source = TermManager.get_term_post_source(term_post['kalla']['id'])
                        term_post['is_source_in_public'] = source is not None
                        changed_term['term'] = term_post

            new_errors = get_errors_with_term_posts(errors)
            new_diffs.append((source_id, source_filename, source_diff, new_errors))
        diffs = new_diffs

    # Group term post rows in other errors.
    if (is_not_empty_list(other_errors)):
        errors_in_file = []
        for other_error in other_errors:
            source_id, file_name, rows = other_error
            error_list = get_errors_with_term_posts(rows)
            errors_in_file.append((source_id, file_name, error_list))
        other_errors = errors_in_file

    context['diffs'] = diffs
    context['other_errors'] = other_errors
    return render(request, 'edit_compare.html', context)


def get_errors_with_term_posts(old_source_errors):
    new_source_errors = []
    if (is_not_empty_list(old_source_errors)):
        index = 0
        while (index < len(old_source_errors)):
            row = old_source_errors[index]
            error_information = {}
            if (row == 'Termpost:'):
                index = index + 1
                error_information['type'] = 'Termpost'
                term_post_fields = []
                error_information['rows'] = term_post_fields
                while (index < len(old_source_errors)):
                    row = old_source_errors[index]
                    if (row == '\t'):
                        break
                    else:
                        term_post_fields.append(row)
                        index = index + 1
            else:
                error_information['type'] = 'error'
                error_information['row'] = row
            new_source_errors.append(error_information)
            index = index + 1
    return new_source_errors


def get_parameter(request, parameter_name, form_parameter_name=None) -> str:
    parameter = None
    if ((request.method == "GET") and (parameter_name in request.GET)):
        parameter = request.GET[parameter_name]
    if (request.method == "POST"):
        if (form_parameter_name is None):
            form_parameter_name = parameter_name
        if (form_parameter_name in request.POST):
            parameter = request.POST[form_parameter_name]
        else:
            if (parameter_name in request.GET):
                parameter = request.GET[parameter_name]
    return parameter


def harvest_to_public():
    TermManager.import_bankvalvet('public', True)


def harvest_to_staging():
    TermManager.import_bankvalvet('staging', True)


def has_parameter(request, parameter_name, form_parameter_name=None) -> bool:
    parameter = get_parameter(request, parameter_name, form_parameter_name)
    return parameter is not None


@login_required
def import_data(request):
    context = {}

    is_action_performed = False
    if (has_parameter(request, 'action', 'action')):
        action = get_parameter(request, 'action', 'action')

        if (action == 'GetNextSourceId'):
            context['next_source_id'] = SourceFileManager.get_next_free_source_id()
            context['action'] = action
            is_action_performed = True

        if (action == 'ImportChangedFiles'):
            file_names = TermManager.import_bankvalvet('staging', False)
            context['source_file_count'] = len(file_names)
            context['source_files'] = file_names
            context['action'] = action
            is_action_performed = True

        elif (action == 'UpdateBankvalvet'):
            update_bank_vault(context)
            context['action'] = action
            is_action_performed = True

    if (not is_action_performed):
        file_names = SourceFileManager.get_file_names()
        context['action'] = 'Start'
        context['source_file_count'] = len(file_names)
        context['source_files'] = file_names

    context['public_term_post_count'] = TermManager.get_term_post_count(Database.PUBLIC_DATABASE_NAME)
    context['staging_term_post_count'] = TermManager.get_term_post_count(Database.STAGING_DATABASE_NAME)
    context['public_source_count'] = TermManager.get_term_post_source_count(True, Database.PUBLIC_DATABASE_NAME)
    context['staging_source_count'] = TermManager.get_term_post_source_count(True, Database.STAGING_DATABASE_NAME)
    return render(request, 'edit_import.html', context)


@login_required
def import_data_all(request):
    context = {}
    file_names = SourceFileManager.get_file_names()
    context['source_file_count'] = len(file_names)
    context['source_files'] = file_names
    context['public_term_post_count'] = TermManager.get_term_post_count(Database.PUBLIC_DATABASE_NAME)
    context['staging_term_post_count'] = TermManager.get_term_post_count(Database.STAGING_DATABASE_NAME)
    context['public_source_count'] = TermManager.get_term_post_source_count(True, Database.PUBLIC_DATABASE_NAME)
    context['staging_source_count'] = TermManager.get_term_post_source_count(True, Database.STAGING_DATABASE_NAME)
    return render(request, 'edit_import_all.html', context)

@login_required
def import_files_to_public(request):
    harvest_thread = threading.Thread(target=harvest_to_public)
    harvest_thread.setDaemon(True)
    harvest_thread.start()
    return redirect(r'/edit/import-all')


@login_required
def import_files_to_staging(request):
    harvest_thread = threading.Thread(target=harvest_to_staging)
    harvest_thread.setDaemon(True)
    harvest_thread.start()
    return redirect(r'/edit/import-all')


@login_required
def import_source_to_public(request):
    context = {}
    # if (request.method != 'POST'):
        # return render(request, 'edit_import_fail.html', context)

    source_id = int(get_parameter(request, "kallid"))
    term_post_source = TermManager.import_source_to_public(source_id)
    if (term_post_source is None):
        return render(request, 'edit_import_fail.html', context)

    return redirect(r'/edit/compare')


@login_required
def edit_help_texts(request):
    context = {}
    context['help_texts'] = []

    help_texts = Help_texts.objects.all()
    for help_text in help_texts:
        context['help_texts'].append(help_text)

    if request.method == "GET":
        if 'page_name' in request.GET:
            request_page_name = request.GET['page_name']
            helpText = Help_texts.objects.filter(page_name=request_page_name).first()
            context['help_text'] = helpText          # lägg till text och metadata till context

    return render(request, 'edit_help_texts.html', context)


@login_required
def edit_update_text(request):
    context = {}

    if request.method == "GET":
        if 'page_name' in request.GET:
            request_page_name = request.GET['page_name']
            helpText = Help_texts.objects.filter(page_name=request_page_name).first()
            if 'page_title' in request.GET and 'helptext' in request.GET:
                helpText.page_title = request.GET['page_title']
                helpText.help_text = request.GET['helptext']
                helpText.save()
                return render(request, 'show_info_page.html?page_name=' + request_page_name, context)
    elif request.method == "POST":
        if 'page_name' in request.POST:
            request_page_name = request.POST['page_name']
            helpText = Help_texts.objects.filter(page_name=request_page_name).first()
            if 'page_title' in request.POST and 'helptext' in request.POST:
                helpText.page_title = request.POST['page_title']
                helpText.help_text = request.POST['helptext']
                helpText.save()
                return redirect('/help/show-info-page?page-name=' + request_page_name)
    else:
        return render(request, 'edit_help_text.html', context)


def render_termpost(termpost):
    seealso = list(filter(None, [TermManager.get_term_post_by_reference(e, termpost["kalla"]["id"]) for e in termpost.get("seealso", [])]))
    seeunder = list(filter(None, [TermManager.get_term_post_by_reference(e, termpost["kalla"]["id"]) for e in termpost.get("seeunder", [])]))
    terms_langauge_codes = set([term["lang"] for term in termpost.get("terms", [])])
    text_langauge_codes = set([text["lang"] for text in termpost.get("texts", [])])
    language_codes = sort_lang_codes(terms_langauge_codes | text_langauge_codes)
    languages = [get_terms_by_language(termpost, language_code) for language_code in language_codes]
    termname = {"term":""}
    if languages:
        termgroups = languages[0].get("termgroups", [])
        if termgroups:
            group = termgroups[0].get("group", [])
            if group:
                termname = group[0]
    illustrations = termpost.get("ILLU", [])
    classifications = termpost.get("classifications", [])
    termpostid = termpost.get("id")
    termpostkalla = termpost["kalla"]

    term_post = {}
    term_post['languages'] = languages
    term_post['seealso'] = seealso
    term_post['seeunder'] = seeunder
    term_post['illustrations'] = illustrations
    term_post['termpostid'] = termpostid
    term_post['kalla'] = termpostkalla
    term_post['iftexts'] = iftexts
    term_post['termname'] = termname
    term_post['classifications'] = classifications
    term_post['current'] = termpost
    return term_post


@login_required
def create_site_map(request):
    # Create a new sitemap.txt
    with open('./sitemap.txt', 'a', encoding="utf8") as file: # a gives 'append'
        sitemap = File(file)
        sitemap.write(r'https://rikstermbanken.se/' + '\n')
        sitemap.write(r'https://rikstermbanken.se/about/published_resources' + '\n')
        sitemap.write(r'https://rikstermbanken.se/help/show-info-page?page-name=help_search' + '\n')
        sitemap.write(r'https://rikstermbanken.se/help/show-info-page?page-name=help_result' + '\n')
        sitemap.write(r'https://rikstermbanken.se/help/show-info-page?page-name=help_termpost' + '\n')
        sitemap.write(r'https://rikstermbanken.se/help/show-info-page?page-name=help_termpost_parts' + '\n')
        sitemap.write(r'https://rikstermbanken.se/help/show-info-page?page-name=help_term_collection' + '\n')
        sitemap.write(r'https://rikstermbanken.se/help/show-info-page?page-name=about_rikstermbanken' + '\n')
        sitemap.write(r'https://rikstermbanken.se/help/show-info-page?page-name=about_publishing' + '\n')
        sitemap.write(r'https://rikstermbanken.se/help/show-info-page?page-name=about_other_resources' + '\n')
        sitemap.write(r'https://rikstermbanken.se/help/show-info-page?page-name=about_contact' + '\n')

        sources = TermManager.get_term_post_sources()
        for source in sources:
            sitemap.write(r'https://rikstermbanken.se/kallor/' + str(source['id']) + '\n')

        # Template for address https://rikstermbanken.se/termposter/2309/slug

        for source in sources:
            terms = TermManager.get_term_posts_by_term_post_source(source['id'],
                                                                   True,
                                                                   Database.PUBLIC_DATABASE_NAME)
            for term in terms:
                f = urlencode({'slug': term['slugs'][0]})
                f = f[5:]
                sitemap.write(r'https://rikstermbanken.se/termposter/' + str(term['kalla']['id']) + r'/' + f + '\n')

    return import_data(request)


def print_stream(stream, name, f):
    for row in stream.decode("utf-8").splitlines():
        print(name, row.rstrip("\n"), file=f)


def update_bank_vault(context):
    TIME_ZONE_STOCKHOLM = pytz.timezone('Europe/Stockholm')

    with open("./update-bankvalvet.log", 'w') as logfile:
        try:
            now = datetime.datetime.now(tz=TIME_ZONE_STOCKHOLM)
            print("===================", file=logfile)
            print(now.strftime("%Y-%m-%d %H:%M:%S"), file=logfile)
            print("===================", file=logfile)
            logfile.flush()

            git_pull = subprocess.run([r"/usr/local/bin/bankvalvet-update"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            # git_pull = subprocess.run(["./update-bankvalvet.sh"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            print_stream(git_pull.stdout, "[git pull stdout]", logfile)
            print_stream(git_pull.stderr, "[git pull stderr]", logfile)
            git_pull.check_returncode()
            logfile.flush()
        except Exception as error:
            print(f"Misslyckades med att hämta uppdaterade filer från bankvalvet: {str(error)}", file=logfile)
            logfile.flush()

    output = []
    with open("./update-bankvalvet.log", encoding="utf-8") as logfile:
        for row in logfile:
            output.append(row)
    context['log_file'] = output
