from django.http import Http404, HttpResponse, HttpResponseNotFound
from django.shortcuts import render
from model.functions import add_render_information, get_formated_text, get_term_post_name, get_terms_by_language
from model.iftexts import iftexts
from model.import_util import sort_lang_codes
from model.term_manager import TermManager
from rikstermbanken_public.models import get_url_parameters
from rikstermbanken_edit.models import Help_texts
from django.conf import settings
from ISOF.configuration import Configuration


def show_info_page(request):
    context = {}
    context['help_texts'] = []

    set_context(request, context)

    helpText = Help_texts.objects.filter(page_name=request.GET['page-name']).first()


    context["breadcrumb_title"] = helpText.page_title
    context['help_texts'].append(helpText)

    return render(request, 'show_info_page.html', context)

"""
def get_help_search(request):
    context = {}
    set_context(request, context)
    context["breadcrumb_title"] = "Sökhjälp"
    return render(request, 'public_help_search.html', context)

def get_help_result(request):
    context = {}
    set_context(request, context)
    context["breadcrumb_title"] = "Träfflistan"
    return render(request, 'public_help_result.html', context)

def get_help_termpost(request):
    context = {}
    set_context(request, context)
    context["breadcrumb_title"] = "Termposten"
    return render(request, 'public_help_termpost.html', context)

def get_help_termpost_parts(request):
    context = {}
    set_context(request, context)
    context["breadcrumb_title"] = "Termpostens delar"
    return render(request, 'public_help_termpost_parts.html', context)
"""
def get_help_term_collection(request):
    context = {}
    set_context(request, context)
    context["breadcrumb_title"] = "Vad är en termsamling?"
    return render(request, 'public_help_term_collection.html', context)
"""
def get_help_explore(request):
    context = {}
    set_context(request, context)
    context["breadcrumb_title"] = "Utforska termposten"
    return render(request, 'public_help_explore.html', context)

def get_help_other(request):
    context = {}
    set_context(request, context)
    context["breadcrumb_title"] = "Andra termbanker"
    return render(request, 'public_help_other.html', context)

def get_help_contribute(request):
    context = {}
    set_context(request, context)
    context["breadcrumb_title"] = "Publicera"
    return render(request, 'public_help_contribute.html', context)
"""


def get_about_published_resources(request):
    context = {}
    set_context(request, context)
    context["breadcrumb_title"] = "Publicerade termsamlingar"

    sources = TermManager.get_term_post_sources_by_term_posts()
    for source in sources:
        source['information'] = source['_id']
    context['sources'] = sources

    return render(request, 'public_about_published_resources.html', context)
"""
def get_about_technology_and_format(request):
    context = {}
    set_context(request, context)
    context["breadcrumb_title"] = "Teknik och format"
    return render(request, 'public_about_technology.html', context)

def get_about_copyright(request):
    context = {}
    set_context(request, context)
    context["breadcrumb_title"] = "Upphovsrätt"
    return render(request, 'public_about_copyright.html', context)

def get_about_history(request):
    context = {}
    set_context(request, context)
    context["breadcrumb_title"] = "Historia"
    return render(request, 'public_about_history.html', context)

def get_about_accessibility(request):
    context = {}
    set_context(request, context)
    context["breadcrumb_title"] = "Tillgänglighet"
    return render(request, 'public_about_accessibility.html', context)

def get_other_resources(request):
    context = {}
    set_context(request, context)
    context["breadcrumb_title"] = "Andra språk- och terminologiresurser"
    return render(request, 'public_about_other_resources.html', context)

def get_about_contact(request):
    context = {}
    set_context(request, context)
    context["breadcrumb_title"] = "Kontakt"
    return render(request, 'public_about_contact.html', context)

def get_about_rikstermbanken(request):
    context = {}
    set_context(request, context)
    context["breadcrumb_title"] = "Om Rikstermbanken"
    return render(request, 'public_about_rikstermbanken.html', context)

def get_about_publishing(request):
    context = {}
    set_context(request, context)
    context["breadcrumb_title"] = "Att publicera en termsamling"
    return render(request, 'public_about_publishing.html', context)
"""


def get_parameter(request, parameter_name, form_parameter_name=None) -> str:
    parameter = None
    if ((request.method == "GET") and (parameter_name in request.GET)):
        parameter = request.GET[parameter_name]
    if (request.method == "POST"):
        if (form_parameter_name is None):
            form_parameter_name = parameter_name
        if (form_parameter_name in request.POST):
            parameter = request.POST[form_parameter_name]
        else:
            if (parameter_name in request.GET):
                parameter = request.GET[parameter_name]
    return parameter


def get_source(request, source_id):
    context = {}
    set_context(request, context)
    context["breadcrumb_title"] = "Källa"

    source = TermManager.get_term_post_source(source_id)
    if (source is None):
        raise Http404(f"Ingen källa har id {source_id}")
    context['source'] = source

    return render(request, 'public_source.html', context)


def get_sources(request):

    context = {}
    set_context(request, context)
    context["breadcrumb_title"] = "Källor"

    sources = TermManager.get_term_post_sources_by_term_posts()
    for source in sources:
        source['information'] = source['_id']
    context['sources'] = sources

    return render(request, 'public_sources.html', context)


def get_start_page(request):
    context = {}
    set_context(request, context)
    return render(request, 'public_index.html', context)


def get_term_post(request, source_id, slug):
    context = {}
    set_context(request, context)

    term_post = TermManager.get_term_post_by_slug(slug, source_id)

    if term_post is None:
        return render(request, 'public_term_post_not_found_404.html', context)
    seealso = list(filter(None, [TermManager.get_term_post_by_reference(e, term_post["kalla"]["id"]) for e in term_post.get("seealso", [])]))
    seeunder = list(filter(None, [TermManager.get_term_post_by_reference(e, term_post["kalla"]["id"]) for e in term_post.get("seeunder", [])]))
    terms_langauge_codes = set([term["lang"] for term in term_post.get("terms", [])])
    text_langauge_codes = set([text["lang"] for text in term_post.get("texts", [])])
    language_codes = sort_lang_codes(terms_langauge_codes | text_langauge_codes)
    languages = [get_terms_by_language(term_post, language_code) for language_code in language_codes]
    term_name = {"term":""}
    if languages:
        termgroups = languages[0].get("termgroups", [])
        if termgroups:
            group = termgroups[0].get("group", [])
            if group:
                term_name = group[0]
    illustrations = term_post.get("ILLU", [])
    classifications = term_post.get("classifications", [])

    resultpage_view = False
    if (has_parameter(request, 'view')):
        resultpage_view = get_parameter(request, 'view') == "resultpage"
    if resultpage_view:
        template = 'public_term_post_view.html'
    else:
        template = 'public_term_post_page.html'

    if (seeunder is not None):
        for term in seeunder:
            get_term_post_name(term['terms'][0])

    if (seealso is not None):
        for term in seealso:
            get_term_post_name(term['terms'][0])

    add_render_information(languages)

    context['term_post_name'] = get_term_post_name(languages[0]['termgroups'][0]['group'][0])
    context['languages'] = languages
    context['seealso'] = seealso
    context['seeunder'] = seeunder
    context['illustrations'] = illustrations
    context['termpostid'] = term_post.get("id")
    context['slug'] = term_post["slugs"][0]
    context['kalla'] = term_post["kalla"]
    context['iftexts'] = iftexts
    context['termname'] = term_name
    context['classifications'] = classifications
    context['resultpage_view'] = resultpage_view
    context['breadcrumbtitle'] = "Termpost"
    context['current'] = "termpost"
    context["breadcrumb_title"] = "Termpost"
    context["is_term_page"] = True

    return render(request, template, context)


def get_term_post_search_result(request):
    context = {}
    set_context(request, context)
    assert has_parameter(request, 'q')
    search_string = get_parameter(request, "q").strip()
    search_type = context['search_type']
    search_langs = get_parameter(request, "searchlang")
    search_langs_request = []
    if has_parameter(request, 'searchlang'):
        search_langs_request = context['request_search_languages']
    def_langs_request = []
    if has_parameter(request, 'deflang'):
        def_langs_request = context['request_definition_languages']
    term_langs_request = []
    if has_parameter(request, 'termlang'):
        term_langs_request = context['request_term_languages']
    term_search_result = TermManager.get_term_posts_by_search_criteria(search_string,
                                                                       search_type,
                                                                       search_langs_request,
                                                                       def_langs_request,
                                                                       term_langs_request)
    text_types = {"definition": "definition", "explanation": "förklaring"}

    for result in term_search_result.results:
        result["match_lang"] = []
        for match in result["matches"]:
            result["match_lang"].append(match["lang"])
        if (("priotext" in result) and (result["priotext"] is not None) and (len(result["priotext"]) > 0)):
            priotext = result["priotext"][0]
            result['search_result_title'] = text_types[priotext['type']]
            result['priotext_paragraphs'] = get_formated_text(priotext['paragraphs'][0], plain_text=True)
            if (len(result['priotext_paragraphs']) > 78):
                result['priotext_paragraphs'] = result['priotext_paragraphs'][0:78] + " ..."

    for result in term_search_result.results:
        result["other_lang"] = []
        for term in result["terms"]:
            if ((term["status"] in ["TE", "SYTE"]) and (term["lang"] not in result["match_lang"])):
                if (term["lang"] not in result["other_lang"]):
                    result["other_lang"].append(term["lang"])
        result["other_lang"].sort()

    for result in term_search_result.results:
        for match_one in result["matches"]:
            for match_two in match_one["matches"]:
                match_two["is_word_in_search_string"] = {}
                words = match_two["text"].split(" ")
                match_string = ''
                for word in words:
                    if (match_string != ''):
                        match_string = match_string + ' '
                    if (search_string.find(word) == -1):
                        match_string = match_string + word
                    else:
                        match_string = match_string + r'<span class="matchingString">' + word + r'</span>'
                match_two["match_string"] = match_string

    # Sökresultat på följande språk
    if (len(term_search_result.term_langs) > 0):
        term_langs = []
        for type, hits in term_search_result.term_langs:
            # Create URL where this language has been removed from search filter.
            url_parameters = context['url_parameters']
            term_parameter = 'termlang=' + type
            if (url_parameters.find('&' + term_parameter) > -1):
                url_parameters = url_parameters.replace('&' + term_parameter, '')
            if (url_parameters.find('?' + term_parameter) > -1):
                if (url_parameters.find('&') > -1):
                    url_parameters = url_parameters.replace(term_parameter + '&', '')
                else:
                    url_parameters = url_parameters.replace('?' + term_parameter, '')
            language = iftexts["lang"]["pl"][type]
            term_langs.append((type, hits, url_parameters, language))
        term_search_result.term_langs = term_langs

    # Definitioner på följande språk
    if (len(term_search_result.definition_langs) > 0):
        definition_langs = []
        for lang, hits in term_search_result.definition_langs:
            # Create URL where this language has been removed from search filter.
            url_parameters = context['url_parameters']
            lang_parameter = 'deflang=' + lang
            if (url_parameters.find('&' + lang_parameter) > -1):
                url_parameters = url_parameters.replace('&' + lang_parameter, '')
            if (url_parameters.find('?' + lang_parameter) > -1):
                if (url_parameters.find('&') > -1):
                    url_parameters = url_parameters.replace(lang_parameter + '&', '')
                else:
                    url_parameters = url_parameters.replace('?' + lang_parameter, '')
            language = iftexts["lang"]["pl"][lang]
            definition_langs.append((lang, hits, url_parameters, language))
        term_search_result.definition_langs = definition_langs

    # Termer finns på följande språk
    if (len(term_search_result.lang_matches) > 0):
        search_langs = []
        for type, hits in term_search_result.lang_matches:
            # Create URL where this language has been removed from search filter.
            url_parameters = context['url_parameters']
            term_parameter = 'searchlang=' + type
            if (url_parameters.find('&' + term_parameter) > -1):
                url_parameters = url_parameters.replace('&' + term_parameter, '')
            if (url_parameters.find('?' + term_parameter) > -1):
                if (url_parameters.find('&') > -1):
                    url_parameters = url_parameters.replace(term_parameter + '&', '')
                else:
                    url_parameters = url_parameters.replace('?' + term_parameter, '')
            language = iftexts["lang"]["pl"][type]
            search_langs.append((type, hits, url_parameters, language))
        term_search_result.lang_matches = search_langs

    context["results"] = term_search_result.results
    context["result_count"] = len(term_search_result.results)
    context["search_string_expressions"] = term_search_result.search_string_expressions
    context["current"] = "search"
    context["more"] = term_search_result.more
    context["too_many"] = term_search_result.too_many
    context["lang_matches"] = term_search_result.lang_matches
    context["lang_matches_count"] = len(term_search_result.lang_matches)
    context["definition_langs"] = term_search_result.definition_langs
    context["definition_langs_count"] = len(term_search_result.definition_langs)
    context["term_langs"] = term_search_result.term_langs
    context["term_langs_count"] = len(term_search_result.term_langs)
    context["breadcrumb_title"] = "Sökresultat"
    context["text_types"] = text_types
    context["search_langs"] = search_langs
    context["search_word"] = search_string
    context["search_langs_request"] = search_langs_request
    context["search_langs_request_count"] = len(search_langs_request)
    context["def_langs_request"] = def_langs_request
    context["def_langs_request_count"] = len(def_langs_request)
    context["term_langs_request"] = term_langs_request
    context["term_langs_request_count"] = len(term_langs_request)

    return render(request, 'public_term_post_search_result.html', context)


def get_robots_txt(request):
    if (Configuration.computer == Configuration.FRIGG_NAME):
        filename = "robots.txt"
        content = 'Sitemap: https://rikstermbanken.se/static/sitemap.txt\n'
        response = HttpResponse(content, content_type='text/plain')
        response['Content-Disposition'] = 'attachment; filename={0}'.format(filename)
        return response
    else:
        return HttpResponseNotFound('<h1>File robots.txt not found</h1>')



def has_parameter(request, parameter_name, form_parameter_name=None) -> bool:
    parameter = get_parameter(request, parameter_name, form_parameter_name)
    return parameter is not None


def set_context(request, context={}):
    if (has_parameter(request, 'q')):
        context['search_word'] = get_parameter(request, 'q')
    if (has_parameter(request, 's')):
        context['search_type'] = get_parameter(request, 's')
    else:
        context['search_type'] = 'termer'

    if (has_parameter(request, 'termlang')):
        # Ugly way to handle multiple occurrences of the same parameter.
        full_path = request.get_full_path()
        languages = full_path.split('termlang=')
        index = 1
        request_term_languages = []
        while index < len(languages):
            request_term_languages.append(languages[index][0:2])
            index += 1
        context['request_term_languages'] = request_term_languages

    if (has_parameter(request, 'searchlang')):
        # Ugly way to handle multiple occurrences of the same parameter.
        full_path = request.get_full_path()
        languages = full_path.split('searchlang=')
        index = 1
        request_search_languages = []
        while index < len(languages):
            request_search_languages.append(languages[index][0:2])
            index += 1
        context['request_search_languages'] = request_search_languages

    if (has_parameter(request, 'deflang')):
        # Ugly way to handle multiple occurrences of the same parameter.
        full_path = request.get_full_path()
        languages = full_path.split('deflang=')
        index = 1
        request_definition_languages = []
        while index < len(languages):
            request_definition_languages.append(languages[index][0:2])
            index += 1
        context['request_definition_languages'] = request_definition_languages

    if (has_parameter(request, 'termlang') or has_parameter(request, 'deflang') or has_parameter(request, 'searchlang')):
        context['show_term_filter'] = "show_term_filter"


    context['url_parameters'] = get_url_parameters(context)
    context["email_address"] = settings.EMAIL_ADDRESS

    context["help_search_url"] = settings.HELP_SEARCH_URL
    context["help_result_url"] = settings.HELP_RESULT_URL
    context["help_termpost_url"] = settings.HELP_TERMPOST_URL
    context["help_termpost_parts_url"] = settings.HELP_TERMPOST_PARTS_URL
    context["help_term_collection_url"] = settings.HELP_TERM_COLLECTION_URL

    context["about_published_resources_url"] = settings.ABOUT_PUBLISHED_RESOURCES_URL
    context["about_other_resources_url"] = settings.ABOUT_OTHER_RESOURCES_URL
    context["about_contact_url"] = settings.ABOUT_CONTACT_URL
    context["about_rikstermbanken_url"] = settings.ABOUT_RIKSTERMBANKEN_URL
    context["about_publishing_url"] = settings.ABOUT_PUBLISHING_URL
